import warnings

from commonroad.common.validity import (is_natural_number, is_positive)
from commonroad.scenario.state import *
from commonroad.visualization.drawable import IDrawable
from commonroad.visualization.renderer import IRenderer
from commonroad.visualization.draw_params import TrajectoryParams, OptionalSpecificOrAllDrawParams, StateParams


class Trajectory(IDrawable):
    """ Class to model the movement of an object over time. The states of the
    trajectory can be either exact or
    uncertain (see :class:`commonroad.scenario.trajectory.State`); however,
    only exact time_step are allowed. """

    def __init__(self, initial_time_step: int, state_list: List[TraceState]):
        """
        :param initial_time_step: initial time step of the trajectory
        :param state_list: ordered sequence of states over time representing
        the trajectory. It is assumed that
        the time discretization between two states matches the time
        discretization of the scenario.
        """
        self.initial_time_step: int = initial_time_step

        self._state_list: Tuple[TraceState] = tuple(self.check_state_list(state_list))

    def check_state_list(self, state_list: List[TraceState]) -> List[TraceState]:
        """
        Checks whether state list is valid.

        :param state_list: state list which should be evaluated
        :return: evaluated state list
        """
        assert isinstance(state_list, list), ('<Trajectory/state_list>: argument state_list of wrong type. '
                                              'Expected type: %s. Got type: %s.' % (list, type(state_list)))
        assert len(state_list) >= 1, ('<Trajectory/state_list>: argument state_list must contain at least one state.'
                                      ' length of state_list: %s.' % len(state_list))
        assert all(isinstance(state, State) for state in state_list), ('<Trajectory/state_list>: element of '
                                                                       'state_list is of wrong type. Expected type: '
                                                                       '%s.' % List[State])
        assert all(is_natural_number(state.time_step) for state in state_list if hasattr(state,
                                                                                         'time_step')), \
            '<Trajectory/state_list>: Element time_step of each state must be an integer.'
        assert all(set(state_list[0].used_attributes) == set(state.used_attributes) for state in state_list), (
                '<Trajectory/state_list>: all states must have the same attributes. Attributes of first state: %s.' %
                state_list[0].attributes)

        assert state_list[0].time_step == self.initial_time_step, \
            f"state_list[0].time_step={state_list[0].time_step} != " \
            f"self.initial_time_step={self.initial_time_step}"

        return state_list

    def __eq__(self, other):
        if not isinstance(other, Trajectory):
            warnings.warn(f"Inequality between Trajectory {repr(self)} and different type {type(other)}")
            return False

        return self._initial_time_step == other.initial_time_step and list(self._state_list) == list(other.state_list)

    def __hash__(self):
        return hash((self._initial_time_step, self._state_list))

    @property
    def initial_time_step(self) -> int:
        """ Initial time step of the trajectory."""
        return self._initial_time_step

    @initial_time_step.setter
    def initial_time_step(self, initial_time_step):
        assert isinstance(initial_time_step, int), (
            '<Trajectory/initial_time_step>: argument initial_time_step of '
            'wrong type. Expected type: %s. Got type: %s.'
            % (int, type(initial_time_step))
        )
        self._initial_time_step = initial_time_step

    @property
    def state_list(self) -> List[TraceState]:
        """ List of states of the trajectory over time."""
        return list(self._state_list)

    @property
    def final_state(self) -> TraceState:
        """ Final state of the trajectory."""
        return self._state_list[-1]

    def state_at_time_step(self, time_step: int) -> Union[TraceState, None]:
        """
        Function to get the state of a trajectory at a specific time instance.

        :param time_step: considered time step
        :return: state of the trajectory at time_step
        """
        state = None
        if (
            self._initial_time_step
            <= time_step
            < self._initial_time_step + len(self._state_list)
        ):
            state = self._state_list[time_step - self._initial_time_step]
        return state

    def states_in_time_interval(self, time_begin: int, time_end: int) -> List[Union[TraceState, None]]:
        """
        Function to get the states of a trajectory at a specific time interval.

        :param time_begin: first considered time step
        :param time_end: last considered time step
        :return: list of states
        """
        assert time_end >= time_begin
        return [self.state_at_time_step(time_step) for time_step in range(time_begin, time_end+1)]

    def translate_rotate(self, translation: np.ndarray, angle: float):
        """ First translates each state of the trajectory, then rotates each state of the trajectory around the
        origin.

        :param translation: translation vector [x_off, y_off] in x- and y-direction
        :param angle: rotation angle in radian (counter-clockwise)
        """
        assert is_real_number_vector(translation, 2), (
            '<Trajectory/translate_rotate>: argument translation is not '
            'a vector of real numbers of length 2.'
        )
        assert is_real_number(angle), (
            '<Trajectory/translate_rotate>: argument angle must be a scalar. '
            'angle = %s' % angle
        )
        assert is_valid_orientation(angle), (
            '<Trajectory/translate_rotate>: argument angle must be within the '
            'interval [-2pi,2pi]. angle = %s' % angle
        )
        new_state_list = []
        for i in range(len(self._state_list)):
            new_state_list.append(self._state_list[i].translate_rotate(translation, angle))
        self._state_list = tuple(new_state_list)

    @classmethod
    def resample_continuous_time_state_list(cls, states: List[TraceState],
                                            time_stamps_cont: np.ndarray,
                                            resampled_dt: float,
                                            num_resampled_states: int,
                                            initial_time_cont: float = 0) -> 'Trajectory':
        """
        This method resamples a given state list with continuous time vector in a fixed time resolution.
        The interpolation is done in a linear fashion.
        :param states: The list of states to interpolate
        :param time_stamps_cont: The vector of continuous time stamps (corresponding to the states)
        :param resampled_dt: Target time step length
        :param num_resampled_states: The resulting number of states. It must hold (t_0+N*dT) \\in time interval
        :param initial_time_cont: The initial continuous time stamp (default 0). It must hold t\\in time interval
        :return: The resampled trajectory
        """
        assert is_positive(
                resampled_dt), '<Trajectory/interpolate_state_list>: Time step size must be a positive number! ' \
                               'dT = {}'.format(resampled_dt)
        assert isinstance(states, list) and all(isinstance(x, State) for x in
                                                states), '<Trajectory/interpolate_state_list>: Provided state list ' \
                                                         'is not in the correct format! State list = {}'.format(
                states)
        assert is_real_number_vector(
                time_stamps_cont), '<Trajectory/interpolate_state_list>: Provided time vector is not in the ' \
                                   'correct format! time = {}'.format(
                time_stamps_cont)
        assert len(states) == len(
                time_stamps_cont), '<Trajectory/interpolate_state_list>: Provided time and state lists do not ' \
                                   'share the same length! Time = {} / States = {}'.format(
                len(time_stamps_cont), len(states))
        assert is_positive(num_resampled_states) and is_natural_number(
                num_resampled_states), '<Trajectory/interpolate_state_list>: Provided state horizon must be a ' \
                                       'positive Integer! N = {}'.format(
                num_resampled_states)
        assert is_real_number(
                initial_time_cont), '<Trajectory/interpolate_state_list>: Provided initial time must be a ' \
                                    'real number! t_0 = {}'.format(
                initial_time_cont)
        assert any(time_stamps_cont <= initial_time_cont) and any(
                initial_time_cont <= time_stamps_cont), '<Trajectory/interpolate_state_list>: Provided initial ' \
                                                        'time is not within time vector! t_0 = {}'.format(
                initial_time_cont)
        assert any(
                initial_time_cont + num_resampled_states * resampled_dt <= time_stamps_cont), \
            '<Trajectory/interpolate_state_list>: Provided end time is not within time vector! t_h = {}'.format(
                initial_time_cont + num_resampled_states * resampled_dt)

        # prepare interpolation by determining all slots with values
        slots = list()
        values = list()
        for s in states[0].attributes:
            # check if state has attribute s
            if getattr(states[0], s) is not None:
                slots.append(s)
                values.append([])

        # create interpolation vector
        t_i = np.arange(initial_time_cont,
                        initial_time_cont + num_resampled_states * resampled_dt + resampled_dt,
                        resampled_dt)
        values_i = list()

        for s in slots:
            values = list()
            multiple = False
            # go through all states
            for x in states:
                if getattr(x, s) is not None:
                    val = getattr(x, s)
                    assert is_real_number(val) or is_real_number_vector(
                            val), '<Trajectory/interpolate_state_list>: Currently, this method only ' \
                                  'supports states with real numbers! val = {}'.format(
                            val)
                    # check if slot is defined for multiple values
                    if not multiple and hasattr(val, 'shape'):
                        if len(val) > 1:
                            multiple = True
                            for i in range(len(val)):
                                values.append([])
                    if multiple:
                        for i, v in enumerate(val):
                            values[i].append(v)

                    else:
                        values.append(val)
                else:
                    raise ValueError(
                            '<Trajectory/interpolate_state_list>: States do not share the same amount of variables!')

            # do the interpolation
            if multiple:
                temp = list()
                for v in values:
                    temp.append(np.interp(t_i, time_stamps_cont, v))
                # stack values again
                values_i.append(np.array(temp).transpose())

            else:
                values_i.append(np.interp(t_i, time_stamps_cont, values))

        state_type = states[0].__class__.__name__
        print(state_type)

        # create new trajectory
        states_new = list()
        for i in range(len(t_i)):
            variables = dict()
            for j, s in enumerate(slots):
                variables[s] = values_i[j][i]
            variables['time_step'] = i

            states_new.append(globals()[state_type](**variables))

        return cls(states_new[0].time_step, states_new)

    def __str__(self):
        traffic_str = '\n'
        traffic_str += 'Initial time step: {} \n'.format(self.initial_time_step)
        traffic_str += 'Number of states: {}\n'.format(len(self.state_list))
        traffic_str += 'State elements: {}'.format(self.state_list[0].attributes)
        return traffic_str

    def draw(self, renderer: IRenderer, draw_params: OptionalSpecificOrAllDrawParams[TrajectoryParams] = None):
        renderer.draw_trajectory(self, draw_params)



import enum
from typing import List, Set
import numpy as np

import commonroad.geometry.transform
from commonroad.common.validity import *
from commonroad.visualization.drawable import IDrawable
from commonroad.visualization.renderer import IRenderer
from commonroad.visualization.draw_params import TrafficSignParams, OptionalSpecificOrAllDrawParams

TRAFFIC_SIGN_VALIDITY_START = {'WARNING_DANGER_SPOT', 'WARNING_RIGHT_BEFORE_LEFT', 'WARNING_STEEP_HILL_DOWNWARDS',
                               'WARNING_SLIPPERY_ROAD', 'WARNING_CONSTRUCTION_SITE', 'WARNING_CROSSING_CYCLIST',
                               'WARNING_ANIMAL_CROSSING_RIGHT', 'RAILWAY', 'PRIORITY_OPPOSITE_DIRECTION',
                               'TURN_RIGHT_AHEAD', 'TURN_LEFT_AHEAD', 'ONEWAY_RIGHT', 'ONEWAY_LEFT',
                               'PRESCRIBED_PASSING_LEFT', 'PRESCRIBED_PASSING_RIGHT', 'BIKEWAY', 'PEDESTRIAN_SIDEWALK',
                               'PEDESTRIAN_ZONE_START', 'BICYCLE_ROAD_START', 'BUS_LANE', 'BAN_ALL_VEHICLES',
                               'BAN_CARS', 'BAN_TRUCKS', 'BAN_BICYCLE', 'BAN_MOTORCYCLE', 'BAN_BUS', 'BAN_PEDESTRIAN',
                               'BAN_CAR_TRUCK_BUS_MOTORCYCLE', 'BAN_VEHICLES_CARRYING_DANGEROUS_GOODS', 'NO_ENTRY',
                               'MAX_WEIGHT', 'MAX_WIDTH', 'MAX_HEIGHT', 'MAX_LENGTH', 'MAX_SPEED',
                               'MAX_SPEED_ZONE_START', 'MIN_SPEED', 'NO_OVERTAKING_START', 'NO_OVERTAKING_TRUCKS_START',
                               'TRAFFIC_CALMED_AREA_START', 'PRIORITY_OVER_ONCOMING', 'TOWN_SIGN', 'TUNNEL',
                               'INTERSTATE_START', 'HIGHWAY_START', 'PEDESTRIANS_CROSSING'}

TRAFFIC_SIGN_WITH_ADDITIONAL_VALUE = {'MAX_WEIGHT', 'MAX_WIDTH', 'MAX_HEIGHT', 'MAX_LENGTH', 'MAX_SPEED',
                                      'MAX_SPEED_ZONE_START', 'MIN_SPEED', 'ADDITION_VALID_FOR_X_METERS',
                                      'ADDITION_VALID_IN_X_KILOMETERS', 'ADDITION_TIME_PERIOD_PERMITTED'}

LEFT_HAND_TRAFFIC = {'AUS', 'JPN', 'HKG', 'IND', 'JEY', 'IMN', 'IRL', 'JAM', 'KEN', 'MLT', 'MYS', 'NPL', 'NZL', 'ZAF',
                     'SGP', 'THA', 'GBR', 'IDN', 'MAC', 'PAK', 'CYP'}


@enum.unique
class SupportedTrafficSignCountry(enum.Enum):
    GERMANY = 'DEU'
    USA = "USA"
    CHINA = "CHN"
    SPAIN = "ESP"
    RUSSIA = "RUS"
    ARGENTINA = "ARG"
    BELGIUM = "BEL"
    FRANCE = "FRA"
    GREECE = "GRC"
    CROATIA = "HRV"
    ITALY = "ITA"
    PUERTO_RICO = "PRI"
    ZAMUNDA = "ZAM"  # default


@enum.unique
class TrafficSignIDGermany(enum.Enum):
    # default traffic sign IDs (similar to German IDs)
    WARNING_DANGER_SPOT = '101'
    WARNING_RIGHT_BEFORE_LEFT = '102'
    WARNING_LEFT_CURVE = '103-10'
    WARNING_RIGHT_CURVE = '103-20'
    WARNING_STEEP_HILL_DOWNWARDS = '108'
    WARNING_SLIPPERY_ROAD = '114'
    WARNING_CONSTRUCTION_SITE = '123'
    WARNING_TRAFFIC_QUEUES_LIKELY = '124'
    WARNING_ONCOMING_TRAFFIC = '125'
    WARNING_TRAFFIC_LIGHTS_AHEAD = '131'
    WARNING_PEDESTRIANS_RIGHT = '133-10'
    WARNING_PEDESTRIANS_LEFT = '133-20'
    WARNING_CROSSING_CYCLIST = '138'
    WARNING_ANIMAL_CROSSING_RIGHT = '142-10'
    WARNING_LOOSE_GRAVEL = '145-50'
    RAILWAY = '201'
    YIELD = '205'
    STOP = '206'
    PRIORITY_OPPOSITE_DIRECTION = '208'
    TURN_RIGHT_AHEAD = '209-10'
    TURN_LEFT_AHEAD = '209-20'
    KEEP_STRAIGHT_AHEAD = '209-30'
    PRESCRIBED_DIRECTION_RIGHT = '211-20'
    ROUNDABOUT = '215'
    ONEWAY_RIGHT = '220-10'
    ONEWAY_LEFT = '220-20'
    PRESCRIBED_PASSING_LEFT = '222-10'
    PRESCRIBED_PASSING_RIGHT = '222-20'
    DO_NOT_DRIVE_ON_SHOULDER_LANE = '223.2'
    DO_NOT_DRIVE_ON_SHOULDER_LANE_2_LANE = '223.2-50'
    DO_NOT_DRIVE_ON_SHOULDER_LANE_3_LANE = '223.2-51'
    BUS_STOP = '224-50'
    BIKEWAY = '237'
    PEDESTRIAN_SIDEWALK = '239'
    PEDESTRIAN_AND_BICYCLE_ROAD = '240'
    PEDESTRIAN_ZONE_START = '242.1'
    PEDESTRIAN_ZONE_END = '242.2'
    BICYCLE_ROAD_START = '244.1'
    BICYCLE_ROAD_END = '244.2'
    BUS_LANE = '245'
    BAN_ALL_VEHICLES = '250'
    BAN_CARS = '251'
    BAN_TRUCKS = '253'
    BAN_BICYCLE = '254'
    BAN_MOTORCYCLE = '255'
    BAN_BUS = '257-54'
    BAN_PEDESTRIAN = '259'
    BAN_CAR_TRUCK_BUS_MOTORCYCLE = '260'
    BAN_VEHICLES_CARRYING_DANGEROUS_GOODS = '261'
    MAX_WEIGHT = '262'
    MAX_WIDTH = '264'
    MAX_HEIGHT = '265'
    MAX_LENGTH = '266'
    NO_ENTRY = '267'
    ENVIRONMENTAL_ZONE_START = '270.1'
    ENVIRONMENTAL_ZONE_END = '270.2'
    U_TURN = '272'
    MAX_SPEED = '274'
    MAX_SPEED_ZONE_START = '274.1'
    MAX_SPEED_ZONE_END = '274.2'
    MIN_SPEED = '275'
    NO_OVERTAKING_START = '276'
    NO_OVERTAKING_TRUCKS_START = '277'
    MAX_SPEED_END = '278'
    NO_OVERTAKING_END = '280'
    NO_OVERTAKING_TRUCKS_END = '281'
    ALL_MAX_SPEED_AND_OVERTAKING_END = '282'
    NO_STOP_START_RIGHT = '283-10'
    NO_STOP_CENTER_RIGHT = '283-30'
    RESTRICTED_STOP_CENTER_RIGHT = '286-30'
    RIGHT_OF_WAY = '301'
    PRIORITY = '306'
    PRIORITY_OVER_ONCOMING = '308'
    TOWN_SIGN = '310'
    TOWN_SIGN_BACK = '311'
    PARKING_AREA = '314'
    PARKING_AREA_LEFT = '314-10'
    PARKING_AREA_RIGHT = '314-20'
    PARKING_AREA_RIGHT_LEFT = '314-30'
    TRAFFIC_CALMED_AREA_START = '325.1'
    TRAFFIC_CALMED_AREA_END = '325.2'
    TUNNEL = '327'
    EMERYGECNY_STOP = '328'
    INTERSTATE_START = '330.1'
    INTERSTATE_END = '330.2'
    HIGHWAY_START = '331.1'
    HIGHWAY_END = '331.2'
    HIGHWAY_EXIT_WITH_PLACE_NAME = '332'
    EXIT_ROUTE = '332.1'
    HIGHWAY_EXIT = '333'
    EXIT_BUILT_UP = '333-21'
    EXIT_GENERAL = '333-22'
    PEDESTRIANS_CROSSING = '350'
    WATER_PROTECTION_ZONE = '354'
    TRAFFIC_ASSISTANTS = '356'
    DEAD_END = '357'
    POLICE = '363'
    EMERGENCY_CALL_STATION = '365-51'
    GAS_STATION = '365-52'
    CAMP_AND_CARAVAN_SITE = '365-60'
    ATTRACTION_POINT = '386.1'
    TOURISTIC_ROUTE = '386.2'
    NEARBY_ATTRACTION_POINT = '386.3'
    HIGHWAY_INTERSECTION = '406-50'
    DIRECTION_ARROW_SIGN_MULTI = '418-20'
    DIRECTION_ARROW_SIGN_SINGLE = '419-20'
    DIRECTION_SIGN_CONSOLIDATED = '434-50'
    EXPRESSWAY_ARROW_DIRECTION = '430-20'
    ARROW_SIGN_POST_POINT_OF_INTEREST_LEFT = '432-10'
    STATION = '432-20'
    GUIDE_SIGN_TABLE = '434'
    ADVANCE_DIRECTION = '438'
    DIRECTIONS_SIGN = '439'
    EXPRESSWAY_ENTRANCE_DIRECTIONS = '440'
    INTERSTATE_ANNOUNCEMENT = '448'
    INTERSTATE_ADVANCE_DIRECTION = '449'
    HIGHWAY_EXIT_AHEAD_100_METER = '450-50'
    HIGHWAY_EXIT_AHEAD_200_METER = '450-51'
    HIGHWAY_EXIT_AHEAD_300_METER = '450-52'
    EXPRESSWAY_EXIT_100_METRES = '450-53'
    EXPRESSWAY_EXIT_200_METRES = '450-54'
    EXPRESSWAY_EXIT_300_METRES = '450-55'
    INTERSTATE_DISTANCE = '453'
    DETOUR_SKETCH = '458'
    DETOUR_STRAIGHT = '455.1-30'
    DETOUR_ON_DEMAND_LEFT = '460-10'
    DETOUR_ON_DEMAND_GET_IN_LEFT_LANE = '460-12'
    DETOUR_ON_DEMAND_ANNOUNCEMENT_RIGHT = '460-20'
    DETOUR_ON_DEMAND_RIGHT = '460-21'
    DETOUR_ON_DEMAND_GET_IN_RIGHT_LANE = '460-22'
    DETOUR_ON_DEMAND_STRAIGHTFORWARD = '460-30'
    TRANSITION_3_LEFT_2_TRANSITIONED = '501-15'
    TRANSITION_1_LEFT_1_STRAIGHT = '501-16'
    TRANSITION_3_RIGHT = '511-22'
    LANE_BOARD_NO_OPPOSITE_TWO_LANES = '521-30'
    THREE_LANES_NO_ONCOMING_LANES = '521-31'
    FOUR_LANES_NO_ONCOMING_LANES = '521-32'
    FIVE_LANES_NO_ONCOMING_LANES = '521-33'
    LANE_BOARD_3_LANES_NO_OPPOSITE_WITH_SIGNS = '525'
    NARROWING_LANES_1_LANE_FROM_RIGHT = '531-10'
    NARROWING_LANES_1_LANE_FROM_LEFT = '531-20'
    NARROWING_LANES_2_LANES_PLUS_1_LEFT = '531-21'
    FOUR_LANES_NO_ONCOMING_TRAFFIC_TWO_RIGHT_LANES_TURN_RIGHT = '533-22'
    MERGING_LANES_1_LANE_PLUS_1_LANE_RIGHT = '550-20'
    BARRIER = '600-35'
    BARRIER_GATE_100_800 = '600-30'
    BARRIER_GATE_100_1200 = '600-31'
    BARRIER_GATE_100_1600 = '600-32'
    BARRIER_GATE_250_1600 = '600-34'
    BARRIER_GATE = '600-38'
    ROAD_WARNING_POST_SCRAPER_BEACON_RIGHT = '605-10'
    ROAD_WARNING_POST_ARROW_BEACON_RIGHT = '605-11'
    ROAD_WARNING_POST_SCRAPER_BEACON_LEFT = '605-20'
    ROAD_WARNING_POST_SCRAPER_BEACON_ARROW_RIGHT = '605-21'
    ROAD_WARNING_POST_GUIDE_UP_THREE_ARROWS = '605-31'
    DIRECTION_SIGN_LEFT_SINGLE = '625-10'
    DIRECTION_SIGN_LEFT_SMALL = '625-11'
    DIRECTION_SIGN_LEFT_MEDIUM = '625-12'
    DIRECTION_SIGN_LEFT_LARGE = '625-13'
    DIRECTION_SIGN_RIGHT_SINGLE = '625-20'
    DIRECTION_SIGN_RIGHT_SMALL = '625-21'
    DIRECTION_SIGN_RIGHT_MEDIUM = '625-22'
    DIRECTION_SIGN_RIGHT_LARGE = '625-23'
    WARNING_PANEL_RIGHT = '626-10'
    WARNING_PANEL_LEFT = '626-20'
    WARNING_PANEL_STRAIGHT_BROAD = '626-30'
    WARNING_PANEL_STRAIGHT_HIGH = '626-31'
    GUIDE_SILL_WITH_GUIDE_BEACON_RIGHT = '628-10'
    GUIDE_RAIL_WITH_GUIDE_BEACON_RIGHT = '629-10'
    GUIDE_PANEL_WITH_GUIDE_BEACON_RIGHT = '629-20'
    GREEN_ARROW = '720'
    ADDITION_LEFT_DIRECTION = '1000'
    ADDITION_LEFT_DIRECTION_1 = '1000-10'
    ADDITION_LEFT_DIRECTION_DANGER_POINT = '1000-11'
    ADDITION_RIGHT_DIRECTION_1 = '1000-20'
    ADDITION_RIGHT_DIRECTION_DANGER_POINT = '1000-21'
    ADDITION_BOTH_DIRECTIONS_HORIZONTAL = '1000-30'
    ADDITION_BOTH_DIRECTIONS_VERTICAL = '1000-31'
    ADDITION_VALID_FOR_X_METERS = '1001-30'
    ADDITION_VALID_FOR_X_KILOMETERS = '1001-31'
    ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_RIGHT_YIELD = '1002-10'
    ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_RIGHT_YIELD = '1002-11'
    ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = '1002-12'
    ADDITION_LEFT_TURNING_PRIORITY_WITH_RIGHT_YIELD = '1002-13'
    ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = '1002-14'
    ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_LEFT_YIELD = '1002-20'
    ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_LEFT_YIELD = '1002-21'
    ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = '1002-22'
    ADDITION_RIGHT_TURNING_PRIORITY_WITH_LEFT_YIELD = '1002-23'
    ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = '1002-24'
    ADDITION_VALID_IN_X_METERS = '1004-30'
    ADDITION_VALID_IN_X_KILOMETERS = '1004-31'
    ADDITION_VALID_IN_200_KILOMETERS = '1004-32'
    ADDITION_VALID_IN_400_METRES = '1004-33'
    ADDITION_VALID_IN_600_METRES = '1004-34'
    ADDITION_VALID_IN_2_KILOMETERS = '1004-35'
    ADDITION_OIL_ON_ROAD = '1006-30'
    ADDITION_SMOKE = '1006-31'
    ADDITION_LOOSE_GRAVEL = '1006-32'
    ADDITION_BUILDING_SITE_EXIT = '1006-33'
    ADDITION_DAMAGED_ROAD = '1006-34'
    ADDITION_DIRTY_ROAD = '1006-35'
    ADDITION_DANGER_OF_COLLISION = '1006-36'
    ADDITION_TOAD_MIGRATION = '1006-37'
    ADDITION_DANGER_OF_CONGESTION = '1006-38'
    ADDITION_RESTRICTED_VIEW_DUE_TO_TREES = '1006-39'
    DANGER_INDICATION_SMOKE = '1007-31'
    ADDITION_CHILDREN_PLAYING_ON_ROAD = '1010-10'
    ADDITION_WINTER_SPORTS_ALLOWED = '1010-11'
    ADDITION_TRAILERS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = '1010-12'
    ADDITION_CARAVANS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = '1010-13'
    ADDITION_ROLLING_HIGHWAY = '1010-14'
    ADDITION_LOADING_AREA = '1012-30'
    ADDITION_END = '1012-31'
    ADDITION_GET_OFF_BICYCLES = '1012-32'
    ADDITION_NO_MOPEDS = '1012-33'
    ADDITION_GREEN_WAVE_AT_KM_H = '1012-34'
    ADDITION_STOP_HERE_AT_RED = '1012-35'
    ADDITION_NOISE_CONTROL = '1012-36'
    ADDITION_INFLOW_REGULATION = '1012-37'
    ADDITION_SECONDARY_LANE = '1012-38'
    ADDITION_SCHOOL = '1012-50'
    ADDITION_KINDERGARTEN = '1012-51'
    ADDITION_RETIREMENT_HOME = '1012-52'
    ADDITION_HOSPITAL = '1012-53'
    ADDITION_RESIDENTS_PERMITTED = '1020-30'
    ADDITION_BICYCLES_PERMITTED = '1022-10'
    ADDITION_CARS_PERMITTED = '1024-10'
    ADDITION_AGRICULTURE_PERMITTED = '1026-36'
    ADDITION_FOREST_PERMITTED = '1026-37'
    ADDITION_AGRICULTURE_FOREST_PERMITTED = '1026-38'
    ADDITION_GREEN_STICKER_PERMITTED = '1031-52'
    ADDITION_TIME_PERIOD_PERMITTED = '1040-30'
    ADDITION_MOTOR_VEHICLES_ALLOWED_MASS_3_5_TONS = '1048-12'
    ADDITION_MIN_MASS_3_5_TONS = '1049-13'
    ADDITION_NO_WATER_POLLUTANTS_LOADED = '1052-31'
    ALLOWED_MASS_7_5_TONS = '1053-33'
    ADDITION_VALID_ON_SHOULDER = '1053-34'
    ADDITION_VALID_WHEN_WET = '1053-35'
    LINE_MARKING_MISSING = '2113'
    UNKNOWN = ''


@enum.unique
class TrafficSignIDZamunda(enum.Enum):  # default traffic sign IDs (similar to German IDs)
    WARNING_DANGER_SPOT = '101'
    WARNING_RIGHT_BEFORE_LEFT = '102'
    WARNING_LEFT_CURVE = '103-10'
    WARNING_RIGHT_CURVE = '103-20'
    WARNING_STEEP_HILL_DOWNWARDS = '108'
    WARNING_SLIPPERY_ROAD = '114'
    WARNING_CONSTRUCTION_SITE = '123'
    WARNING_TRAFFIC_QUEUES_LIKELY = '124'
    WARNING_ONCOMING_TRAFFIC = '125'
    WARNING_TRAFFIC_LIGHTS_AHEAD = '131'
    WARNING_PEDESTRIANS_RIGHT = '133-10'
    WARNING_PEDESTRIANS_LEFT = '133-20'
    WARNING_CROSSING_CYCLIST = '138'
    WARNING_ANIMAL_CROSSING_RIGHT = '142-10'
    WARNING_LOOSE_GRAVEL = '145-50'
    RAILWAY = '201'
    YIELD = '205'
    STOP = '206'
    PRIORITY_OPPOSITE_DIRECTION = '208'
    TURN_RIGHT_AHEAD = '209-10'
    TURN_LEFT_AHEAD = '209-20'
    KEEP_STRAIGHT_AHEAD = '209-30'
    PRESCRIBED_DIRECTION_RIGHT = '211-20'
    ROUNDABOUT = '215'
    ONEWAY_RIGHT = '220-10'
    ONEWAY_LEFT = '220-20'
    PRESCRIBED_PASSING_LEFT = '222-10'
    PRESCRIBED_PASSING_RIGHT = '222-20'
    DO_NOT_DRIVE_ON_SHOULDER_LANE = '223.2'
    DO_NOT_DRIVE_ON_SHOULDER_LANE_2_LANE = '223.2-50'
    DO_NOT_DRIVE_ON_SHOULDER_LANE_3_LANE = '223.2-51'
    BUS_STOP = '224-50'
    BIKEWAY = '237'
    PEDESTRIAN_SIDEWALK = '239'
    PEDESTRIAN_AND_BICYCLE_ROAD = '240'
    PEDESTRIAN_ZONE_START = '242.1'
    PEDESTRIAN_ZONE_END = '242.2'
    BICYCLE_ROAD_START = '244.1'
    BICYCLE_ROAD_END = '244.2'
    BUS_LANE = '245'
    BAN_ALL_VEHICLES = '250'
    BAN_CARS = '251'
    BAN_TRUCKS = '253'
    BAN_BICYCLE = '254'
    BAN_MOTORCYCLE = '255'
    BAN_BUS = '257-54'
    BAN_PEDESTRIAN = '259'
    BAN_CAR_TRUCK_BUS_MOTORCYCLE = '260'
    BAN_VEHICLES_CARRYING_DANGEROUS_GOODS = '261'
    MAX_WEIGHT = '262'
    MAX_WIDTH = '264'
    MAX_HEIGHT = '265'
    MAX_LENGTH = '266'
    NO_ENTRY = '267'
    ENVIRONMENTAL_ZONE_START = '270.1'
    ENVIRONMENTAL_ZONE_END = '270.2'
    U_TURN = '272'
    MAX_SPEED = '274'
    MAX_SPEED_ZONE_START = '274.1'
    MAX_SPEED_ZONE_END = '274.2'
    MIN_SPEED = '275'
    NO_OVERTAKING_START = '276'
    NO_OVERTAKING_TRUCKS_START = '277'
    MAX_SPEED_END = '278'
    NO_OVERTAKING_END = '280'
    NO_OVERTAKING_TRUCKS_END = '281'
    ALL_MAX_SPEED_AND_OVERTAKING_END = '282'
    NO_STOP_START_RIGHT = '283-10'
    NO_STOP_CENTER_RIGHT = '283-30'
    RESTRICTED_STOP_CENTER_RIGHT = '286-30'
    RIGHT_OF_WAY = '301'
    PRIORITY = '306'
    PRIORITY_OVER_ONCOMING = '308'
    TOWN_SIGN = '310'
    TOWN_SIGN_BACK = '311'
    PARKING_AREA = '314'
    PARKING_AREA_LEFT = '314-10'
    PARKING_AREA_RIGHT = '314-20'
    PARKING_AREA_RIGHT_LEFT = '314-30'
    TRAFFIC_CALMED_AREA_START = '325.1'
    TRAFFIC_CALMED_AREA_END = '325.2'
    TUNNEL = '327'
    EMERYGECNY_STOP = '328'
    INTERSTATE_START = '330.1'
    INTERSTATE_END = '330.2'
    HIGHWAY_START = '331.1'
    HIGHWAY_END = '331.2'
    HIGHWAY_EXIT_WITH_PLACE_NAME = '332'
    EXIT_ROUTE = '332.1'
    HIGHWAY_EXIT = '333'
    EXIT_BUILT_UP = '333-21'
    EXIT_GENERAL = '333-22'
    PEDESTRIANS_CROSSING = '350'
    WATER_PROTECTION_ZONE = '354'
    TRAFFIC_ASSISTANTS = '356'
    DEAD_END = '357'
    POLICE = '363'
    EMERGENCY_CALL_STATION = '365-51'
    GAS_STATION = '365-52'
    CAMP_AND_CARAVAN_SITE = '365-60'
    ATTRACTION_POINT = '386.1'
    TOURISTIC_ROUTE = '386.2'
    NEARBY_ATTRACTION_POINT = '386.3'
    HIGHWAY_INTERSECTION = '406-50'
    DIRECTION_ARROW_SIGN_MULTI = '418-20'
    DIRECTION_ARROW_SIGN_SINGLE = '419-20'
    DIRECTION_SIGN_CONSOLIDATED = '434-50'
    EXPRESSWAY_ARROW_DIRECTION = '430-20'
    ARROW_SIGN_POST_POINT_OF_INTEREST_LEFT = '432-10'
    STATION = '432-20'
    GUIDE_SIGN_TABLE = '434'
    ADVANCE_DIRECTION = '438'
    DIRECTIONS_SIGN = '439'
    EXPRESSWAY_ENTRANCE_DIRECTIONS = '440'
    INTERSTATE_ANNOUNCEMENT = '448'
    INTERSTATE_ADVANCE_DIRECTION = '449'
    HIGHWAY_EXIT_AHEAD_100_METER = '450-50'
    HIGHWAY_EXIT_AHEAD_200_METER = '450-51'
    HIGHWAY_EXIT_AHEAD_300_METER = '450-52'
    EXPRESSWAY_EXIT_100_METRES = '450-53'
    EXPRESSWAY_EXIT_200_METRES = '450-54'
    EXPRESSWAY_EXIT_300_METRES = '450-55'
    INTERSTATE_DISTANCE = '453'
    DETOUR_SKETCH = '458'
    DETOUR_STRAIGHT = '455.1-30'
    DETOUR_ON_DEMAND_LEFT = '460-10'
    DETOUR_ON_DEMAND_GET_IN_LEFT_LANE = '460-12'
    DETOUR_ON_DEMAND_ANNOUNCEMENT_RIGHT = '460-20'
    DETOUR_ON_DEMAND_RIGHT = '460-21'
    DETOUR_ON_DEMAND_GET_IN_RIGHT_LANE = '460-22'
    DETOUR_ON_DEMAND_STRAIGHTFORWARD = '460-30'
    TRANSITION_3_LEFT_2_TRANSITIONED = '501-15'
    TRANSITION_1_LEFT_1_STRAIGHT = '501-16'
    TRANSITION_3_RIGHT = '511-22'
    LANE_BOARD_NO_OPPOSITE_TWO_LANES = '521-30'
    THREE_LANES_NO_ONCOMING_LANES = '521-31'
    FOUR_LANES_NO_ONCOMING_LANES = '521-32'
    FIVE_LANES_NO_ONCOMING_LANES = '521-33'
    LANE_BOARD_3_LANES_NO_OPPOSITE_WITH_SIGNS = '525'
    NARROWING_LANES_1_LANE_FROM_RIGHT = '531-10'
    NARROWING_LANES_1_LANE_FROM_LEFT = '531-20'
    NARROWING_LANES_2_LANES_PLUS_1_LEFT = '531-21'
    FOUR_LANES_NO_ONCOMING_TRAFFIC_TWO_RIGHT_LANES_TURN_RIGHT = '533-22'
    MERGING_LANES_1_LANE_PLUS_1_LANE_RIGHT = '550-20'
    BARRIER = '600-35'
    BARRIER_GATE_100_800 = '600-30'
    BARRIER_GATE_100_1200 = '600-31'
    BARRIER_GATE_100_1600 = '600-32'
    BARRIER_GATE_250_1600 = '600-34'
    BARRIER_GATE = '600-38'
    ROAD_WARNING_POST_SCRAPER_BEACON_RIGHT = '605-10'
    ROAD_WARNING_POST_ARROW_BEACON_RIGHT = '605-11'
    ROAD_WARNING_POST_SCRAPER_BEACON_LEFT = '605-20'
    ROAD_WARNING_POST_SCRAPER_BEACON_ARROW_RIGHT = '605-21'
    ROAD_WARNING_POST_GUIDE_UP_THREE_ARROWS = '605-31'
    DIRECTION_SIGN_LEFT_SINGLE = '625-10'
    DIRECTION_SIGN_LEFT_SMALL = '625-11'
    DIRECTION_SIGN_LEFT_MEDIUM = '625-12'
    DIRECTION_SIGN_LEFT_LARGE = '625-13'
    DIRECTION_SIGN_RIGHT_SINGLE = '625-20'
    DIRECTION_SIGN_RIGHT_SMALL = '625-21'
    DIRECTION_SIGN_RIGHT_MEDIUM = '625-22'
    DIRECTION_SIGN_RIGHT_LARGE = '625-23'
    WARNING_PANEL_RIGHT = '626-10'
    WARNING_PANEL_LEFT = '626-20'
    WARNING_PANEL_STRAIGHT_BROAD = '626-30'
    WARNING_PANEL_STRAIGHT_HIGH = '626-31'
    GUIDE_SILL_WITH_GUIDE_BEACON_RIGHT = '628-10'
    GUIDE_RAIL_WITH_GUIDE_BEACON_RIGHT = '629-10'
    GUIDE_PANEL_WITH_GUIDE_BEACON_RIGHT = '629-20'
    GREEN_ARROW = '720'
    ADDITION_LEFT_DIRECTION = '1000'
    ADDITION_LEFT_DIRECTION_1 = '1000-10'
    ADDITION_LEFT_DIRECTION_DANGER_POINT = '1000-11'
    ADDITION_RIGHT_DIRECTION_1 = '1000-20'
    ADDITION_RIGHT_DIRECTION_DANGER_POINT = '1000-21'
    ADDITION_BOTH_DIRECTIONS_HORIZONTAL = '1000-30'
    ADDITION_BOTH_DIRECTIONS_VERTICAL = '1000-31'
    ADDITION_VALID_FOR_X_METERS = '1001-30'
    ADDITION_VALID_FOR_X_KILOMETERS = '1001-31'
    ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_RIGHT_YIELD = '1002-10'
    ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_RIGHT_YIELD = '1002-11'
    ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = '1002-12'
    ADDITION_LEFT_TURNING_PRIORITY_WITH_RIGHT_YIELD = '1002-13'
    ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = '1002-14'
    ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_LEFT_YIELD = '1002-20'
    ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_LEFT_YIELD = '1002-21'
    ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = '1002-22'
    ADDITION_RIGHT_TURNING_PRIORITY_WITH_LEFT_YIELD = '1002-23'
    ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = '1002-24'
    ADDITION_VALID_IN_X_METERS = '1004-30'
    ADDITION_VALID_IN_X_KILOMETERS = '1004-31'
    ADDITION_VALID_IN_200_KILOMETERS = '1004-32'
    ADDITION_VALID_IN_400_METRES = '1004-33'
    ADDITION_VALID_IN_600_METRES = '1004-34'
    ADDITION_VALID_IN_2_KILOMETERS = '1004-35'
    ADDITION_OIL_ON_ROAD = '1006-30'
    ADDITION_SMOKE = '1006-31'
    ADDITION_LOOSE_GRAVEL = '1006-32'
    ADDITION_BUILDING_SITE_EXIT = '1006-33'
    ADDITION_DAMAGED_ROAD = '1006-34'
    ADDITION_DIRTY_ROAD = '1006-35'
    ADDITION_DANGER_OF_COLLISION = '1006-36'
    ADDITION_TOAD_MIGRATION = '1006-37'
    ADDITION_DANGER_OF_CONGESTION = '1006-38'
    ADDITION_RESTRICTED_VIEW_DUE_TO_TREES = '1006-39'
    DANGER_INDICATION_SMOKE = '1007-31'
    ADDITION_CHILDREN_PLAYING_ON_ROAD = '1010-10'
    ADDITION_WINTER_SPORTS_ALLOWED = '1010-11'
    ADDITION_TRAILERS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = '1010-12'
    ADDITION_CARAVANS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = '1010-13'
    ADDITION_ROLLING_HIGHWAY = '1010-14'
    ADDITION_LOADING_AREA = '1012-30'
    ADDITION_END = '1012-31'
    ADDITION_GET_OFF_BICYCLES = '1012-32'
    ADDITION_NO_MOPEDS = '1012-33'
    ADDITION_GREEN_WAVE_AT_KM_H = '1012-34'
    ADDITION_STOP_HERE_AT_RED = '1012-35'
    ADDITION_NOISE_CONTROL = '1012-36'
    ADDITION_INFLOW_REGULATION = '1012-37'
    ADDITION_SECONDARY_LANE = '1012-38'
    ADDITION_SCHOOL = '1012-50'
    ADDITION_KINDERGARTEN = '1012-51'
    ADDITION_RETIREMENT_HOME = '1012-52'
    ADDITION_HOSPITAL = '1012-53'
    ADDITION_RESIDENTS_PERMITTED = '1020-30'
    ADDITION_BICYCLES_PERMITTED = '1022-10'
    ADDITION_CARS_PERMITTED = '1024-10'
    ADDITION_AGRICULTURE_PERMITTED = '1026-36'
    ADDITION_FOREST_PERMITTED = '1026-37'
    ADDITION_AGRICULTURE_FOREST_PERMITTED = '1026-38'
    ADDITION_GREEN_STICKER_PERMITTED = '1031-52'
    ADDITION_TIME_PERIOD_PERMITTED = '1040-30'
    ADDITION_MOTOR_VEHICLES_ALLOWED_MASS_3_5_TONS = '1048-12'
    ADDITION_MIN_MASS_3_5_TONS = '1049-13'
    ADDITION_NO_WATER_POLLUTANTS_LOADED = '1052-31'
    ALLOWED_MASS_7_5_TONS = '1053-33'
    ADDITION_VALID_ON_SHOULDER = '1053-34'
    ADDITION_VALID_WHEN_WET = '1053-35'
    LINE_MARKING_MISSING = '2113'
    UNKNOWN = ''


@enum.unique
class TrafficSignIDUsa(enum.Enum):
    MAX_SPEED = 'R2-1'
    U_TURN = 'R3-4'
    ROAD_WORK_AHEAD = 'CW20-1'
    NO_PARKING_ANY_TIME = 'R7-1'
    NO_STANDING = 'R7-4'
    TOW_AWAY_ZONE = 'R7-201a'
    ONE_WAY_LEFT = 'R6-1L'
    ONE_WAY_RIGHT = 'R6-1R'
    DO_NOT_ENTER = 'R5-1'
    NO_LEFT_TURN = 'R3-2'
    RIGHT_TURN_ONLY = 'R3-5R'
    TURN_ONLY_LANES = 'R3-8b'
    NO_RIGHT_TURN = 'R3-1'
    KEEP_RIGHT = 'R4-7'
    SIGNAL_AHEAD = 'W3-3'
    LOADING_ZONE = 'R8-3gP'
    NO_PARKING = 'R8-3'
    LEFT_TURN_ONLY = 'R3-5L'
    NO_STRAIGHT_THROUGH = 'R3-27'
    LEFT_REVERSE_TURN = 'W1-3L'
    PEDESTRIAN_WARNING = 'W11-2'
    DIAGONAL_DOWNWARD_LEFT_ARROW = 'M6-2aL'
    RIGHT_LANE_ENDS = 'W4-2R'
    RESERVED_HANDICAP_PARKING = 'R7-8'
    NO_PARKING_BUS_STOP = 'R7-107'
    ON_PAVEMENT = 'R8-3C'
    DO_NOT_BLOCK_INTERSECTION = 'R10-7'
    WARNING_ARROW_LEFT = 'W1-6L'
    UNKNOWN = ''


@enum.unique
class TrafficSignIDChina(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''


@enum.unique
class TrafficSignIDSpain(enum.Enum):
    YIELD = 'r1'
    STOP = 'r2'
    BAN_ALL_VEHICLES = 'r100'
    NO_ENTRY = 'r101'
    BAN_TRUCKS = 'r106'
    MAX_WEIGHT = 'r107'
    MAX_HEIGHT = 'r205'
    MAX_SPEED = 'r301'
    NO_OVERTAKING_START = 'r305'
    NO_STOPPING = 'r307'
    NO_WAITING = 'r308'
    PEDESTRIANS_CROSSING = 's13'
    UNKNOWN = ''


@enum.unique
class TrafficSignIDRussia(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''  # TODO: add actual IDs


@enum.unique
class TrafficSignIDArgentina(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''  # TODO: add actual IDs


@enum.unique
class TrafficSignIDBelgium(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''  # TODO: add actual IDs


@enum.unique
class TrafficSignIDFrance(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''  # TODO: add actual IDs


@enum.unique
class TrafficSignIDGreece(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''  # TODO: add actual IDs


@enum.unique
class TrafficSignIDCroatia(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''


@enum.unique
class TrafficSignIDItaly(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''


@enum.unique
class TrafficSignIDPuertoRico(enum.Enum):
    MAX_SPEED = '274'  # TODO: change to actual ID
    UNKNOWN = ''  # TODO: add actual IDs


class TrafficSignElement:
    """ Class which represents a collection of traffic signs at one position"""

    def __init__(self, traffic_sign_element_id: Union[
        TrafficSignIDZamunda, TrafficSignIDUsa, TrafficSignIDSpain, TrafficSignIDGermany, TrafficSignIDChina,
        TrafficSignIDRussia],
                 additional_values: List[str] = []):
        """

        :param traffic_sign_element_id: ID of traffic sign element (must be element of a traffic sign element enum)
        :param additional_values: list of additional values of a traffic sign element, e.g. velocity, time, city name
        """
        self._traffic_sign_element_id = traffic_sign_element_id
        self._additional_values = additional_values

    def __eq__(self, other):
        if not isinstance(other, TrafficSignElement):
            warnings.warn(f"Inequality between TrafficSignElement {repr(self)} and different type {type(other)}")
            return False

        if self.traffic_sign_element_id == other.traffic_sign_element_id \
                and set(self.additional_values) == set(other.additional_values):
            return True

        warnings.warn(f"Inequality of TrafficSignElement {repr(self)} and the other one {repr(other)}")
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        return hash((self._traffic_sign_element_id, frozenset(self.additional_values)))

    def __str__(self):
        return f"TrafficSignElement with id {self._traffic_sign_element_id} and values {self._additional_values}"

    def __repr__(self):
        return f"TrafficSignElement(traffic_sign_element_id={self._traffic_sign_element_id}, " \
               f"additional_values={self._additional_values})"

    @property
    def traffic_sign_element_id(self) -> enum:
        return self._traffic_sign_element_id

    @traffic_sign_element_id.setter
    def traffic_sign_element_id(self, traffic_sign_element_id: enum):
        self._traffic_sign_element_id = traffic_sign_element_id

    @property
    def additional_values(self) -> List[str]:
        return self._additional_values

    @additional_values.setter
    def additional_values(self, additional_values: List[str]):
        self._additional_values = additional_values


class TrafficSign(IDrawable):
    """Class to represent a traffic sign"""

    def __init__(self, traffic_sign_id: int, traffic_sign_elements: List[TrafficSignElement],
                 first_occurrence: Set[int], position: np.ndarray, virtual: bool = False):
        """
        :param traffic_sign_id: ID of traffic sign
        :param traffic_sign_elements: list of traffic sign elements
        :param first_occurrence: lanelet ID where traffic sign first appears
        :param position: position of traffic sign
        :param virtual: boolean indicating if this traffic sign is also
        placed there in the real environment, or it
        is added for other reasons (e.g., completeness of scenario)
        """
        self._traffic_sign_id = traffic_sign_id
        self._position = position
        self._traffic_sign_elements = traffic_sign_elements
        self._virtual = virtual
        self._first_occurrence = first_occurrence

    def __eq__(self, other):
        if not isinstance(other, TrafficSign):
            warnings.warn(f"Inequality between TrafficSign {repr(self)} and different type {type(other)}")
            return False

        list_elements_eq = True
        traffic_sign_elements = {traffic_sign_element.traffic_sign_element_id: traffic_sign_element
                                 for traffic_sign_element in self._traffic_sign_elements}
        traffic_sign_elements_other = {traffic_sign_element.traffic_sign_element_id: traffic_sign_element
                                       for traffic_sign_element in other._traffic_sign_elements}
        traffic_sign_eq = len(traffic_sign_elements) == len(traffic_sign_elements_other)
        for k in traffic_sign_elements.keys():
            if k not in traffic_sign_elements_other:
                traffic_sign_eq = False
                continue
            if traffic_sign_elements.get(k) != traffic_sign_elements_other.get(k):
                list_elements_eq = False

        position_string = np.array2string(np.around(self._position.astype(float), 10), precision=10)
        position_other_string = np.array2string(np.around(other.position.astype(float), 10), precision=10)

        if traffic_sign_eq and self._traffic_sign_id == other.traffic_sign_id \
                and position_string == position_other_string and self._virtual == other.virtual \
                and self._first_occurrence == other.first_occurrence:
            return list_elements_eq

        warnings.warn(f"Inequality of TrafficSign {repr(self)} and the other one {repr(other)}")
        return False

    def __hash__(self):
        position_string = np.array2string(np.around(self._position.astype(float), 10), precision=10)
        return hash((self._traffic_sign_id, position_string, frozenset(self._traffic_sign_elements), self._virtual,
                     frozenset(self._first_occurrence)))

    def __str__(self):
        return f"TrafficSign with id {self._traffic_sign_id} placed at {self._position}"

    def __repr__(self):
        return f"TrafficSign(traffic_sign_id={self._traffic_sign_id}, " \
               f"traffic_sign_elements={repr(self._traffic_sign_elements)}, " \
               f"first_occurrence={self._first_occurrence}, " \
               f"position={self._position.tolist()}, virtual={self._virtual})"

    @property
    def traffic_sign_id(self) -> int:
        return self._traffic_sign_id

    @traffic_sign_id.setter
    def traffic_sign_id(self, traffic_sign_id: int):
        self._traffic_sign_id = traffic_sign_id

    @property
    def position(self) -> np.ndarray:
        return self._position

    @position.setter
    def position(self, position: np.ndarray):
        self._position = position

    @property
    def traffic_sign_elements(self) -> List[TrafficSignElement]:
        return self._traffic_sign_elements

    @traffic_sign_elements.setter
    def traffic_sign_elements(self, traffic_sign_elements: List[TrafficSignElement]):
        self._traffic_sign_elements = traffic_sign_elements

    @property
    def virtual(self) -> bool:
        return self._virtual

    @virtual.setter
    def virtual(self, virtual: bool):
        self._virtual = virtual

    @property
    def first_occurrence(self) -> Set[int]:
        return self._first_occurrence

    @first_occurrence.setter
    def first_occurrence(self, first_occurrence: Set[int]):
        self._first_occurrence = first_occurrence

    def translate_rotate(self, translation: np.ndarray, angle: float):
        """
        This method translates and rotates a traffic sign

        :param translation: The translation given as [x_off,y_off] for the x and y translation
        :param angle: The rotation angle in radian (counter-clockwise defined)
        """

        assert is_real_number_vector(translation, 2), '<TrafficSign/translate_rotate>: argument translation is ' \
                                                      'not a vector of real ' \
                                                      'numbers of length 2.'
        assert is_real_number(angle), '<TrafficSign/translate_rotate>: argument angle must be a ' \
                                      'scalar. ' \
                                      'angle = %s' % angle
        assert is_valid_orientation(angle), '<TrafficSign/translate_rotate>: argument angle must ' \
                                            'be ' \
                                            'within the ' \
                                            'interval [-2pi, 2pi]. angle = %s' % angle
        self._position = commonroad.geometry.transform.translate_rotate(np.array([self._position]), translation, angle)[
            0]

    def draw(self, renderer: IRenderer, draw_params: OptionalSpecificOrAllDrawParams[TrafficSignParams] = None):
        renderer.draw_traffic_light_sign(self, draw_params)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lanelet.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.util_pb2 as util__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='lanelet.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\rlanelet.proto\x12\ncommonroad\x1a\nutil.proto\"w\n\x0fLineMarkingEnum\"d\n\x0bLineMarking\x12\n\n\x06\x44\x41SHED\x10\x00\x12\t\n\x05SOLID\x10\x01\x12\x10\n\x0c\x42ROAD_DASHED\x10\x02\x12\x0f\n\x0b\x42ROAD_SOLID\x10\x03\x12\x0b\n\x07UNKNOWN\x10\x04\x12\x0e\n\nNO_MARKING\x10\x05\"6\n\x0e\x44rivingDirEnum\"$\n\nDrivingDir\x12\x08\n\x04SAME\x10\x00\x12\x0c\n\x08OPPOSITE\x10\x01\"\xbd\x02\n\x0fLaneletTypeEnum\"\xa9\x02\n\x0bLaneletType\x12\t\n\x05URBAN\x10\x00\x12\x0b\n\x07\x43OUNTRY\x10\x01\x12\x0b\n\x07HIGHWAY\x10\x02\x12\r\n\tDRIVE_WAY\x10\x03\x12\x15\n\x11MAIN_CARRIAGE_WAY\x10\x04\x12\x0f\n\x0b\x41\x43\x43\x45SS_RAMP\x10\x05\x12\r\n\tEXIT_RAMP\x10\x06\x12\x0c\n\x08SHOULDER\x10\x07\x12\x0c\n\x08\x42US_LANE\x10\x08\x12\x0c\n\x08\x42US_STOP\x10\t\x12\x10\n\x0c\x42ICYCLE_LANE\x10\n\x12\x0c\n\x08SIDEWALK\x10\x0b\x12\r\n\tCROSSWALK\x10\x0c\x12\x0e\n\nINTERSTATE\x10\r\x12\x10\n\x0cINTERSECTION\x10\x0e\x12\n\n\x06\x42ORDER\x10\x0f\x12\x0b\n\x07PARKING\x10\x10\x12\x0e\n\nRESTRICTED\x10\x11\x12\x0b\n\x07UNKNOWN\x10\x12\"\x9d\x01\n\x0cRoadUserEnum\"\x8c\x01\n\x08RoadUser\x12\x0b\n\x07VEHICLE\x10\x00\x12\x07\n\x03\x43\x41R\x10\x01\x12\t\n\x05TRUCK\x10\x02\x12\x07\n\x03\x42US\x10\x03\x12\x14\n\x10PRIORITY_VEHICLE\x10\x04\x12\x0e\n\nMOTORCYCLE\x10\x05\x12\x0b\n\x07\x42ICYCLE\x10\x06\x12\x0e\n\nPEDESTRIAN\x10\x07\x12\t\n\x05TRAIN\x10\x08\x12\x08\n\x04TAXI\x10\t\"i\n\x05\x42ound\x12!\n\x06points\x18\x01 \x03(\x0b\x32\x11.commonroad.Point\x12=\n\x0cline_marking\x18\x02 \x01(\x0e\x32\'.commonroad.LineMarkingEnum.LineMarking\"\xa3\x01\n\x08StopLine\x12!\n\x06points\x18\x01 \x03(\x0b\x32\x11.commonroad.Point\x12=\n\x0cline_marking\x18\x02 \x02(\x0e\x32\'.commonroad.LineMarkingEnum.LineMarking\x12\x19\n\x11traffic_sign_refs\x18\x03 \x03(\r\x12\x1a\n\x12traffic_light_refs\x18\x04 \x03(\r\"\xe2\x04\n\x07Lanelet\x12\x12\n\nlanelet_id\x18\x01 \x02(\r\x12%\n\nleft_bound\x18\x02 \x02(\x0b\x32\x11.commonroad.Bound\x12&\n\x0bright_bound\x18\x03 \x02(\x0b\x32\x11.commonroad.Bound\x12\x14\n\x0cpredecessors\x18\x04 \x03(\r\x12\x12\n\nsuccessors\x18\x05 \x03(\r\x12\x15\n\radjacent_left\x18\x06 \x01(\r\x12\x16\n\x0e\x61\x64jacent_right\x18\x07 \x01(\r\x12@\n\x11\x61\x64jacent_left_dir\x18\x08 \x01(\x0e\x32%.commonroad.DrivingDirEnum.DrivingDir\x12\x41\n\x12\x61\x64jacent_right_dir\x18\t \x01(\x0e\x32%.commonroad.DrivingDirEnum.DrivingDir\x12\'\n\tstop_line\x18\n \x01(\x0b\x32\x14.commonroad.StopLine\x12>\n\rlanelet_types\x18\x0b \x03(\x0e\x32\'.commonroad.LaneletTypeEnum.LaneletType\x12\x37\n\x0cuser_one_way\x18\x0c \x03(\x0e\x32!.commonroad.RoadUserEnum.RoadUser\x12=\n\x12user_bidirectional\x18\r \x03(\x0e\x32!.commonroad.RoadUserEnum.RoadUser\x12\x19\n\x11traffic_sign_refs\x18\x0e \x03(\r\x12\x1a\n\x12traffic_light_refs\x18\x0f \x03(\r'
  ,
  dependencies=[util__pb2.DESCRIPTOR,])



_LINEMARKINGENUM_LINEMARKING = _descriptor.EnumDescriptor(
  name='LineMarking',
  full_name='commonroad.LineMarkingEnum.LineMarking',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DASHED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SOLID', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BROAD_DASHED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BROAD_SOLID', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NO_MARKING', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=60,
  serialized_end=160,
)
_sym_db.RegisterEnumDescriptor(_LINEMARKINGENUM_LINEMARKING)

_DRIVINGDIRENUM_DRIVINGDIR = _descriptor.EnumDescriptor(
  name='DrivingDir',
  full_name='commonroad.DrivingDirEnum.DrivingDir',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SAME', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OPPOSITE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=180,
  serialized_end=216,
)
_sym_db.RegisterEnumDescriptor(_DRIVINGDIRENUM_DRIVINGDIR)

_LANELETTYPEENUM_LANELETTYPE = _descriptor.EnumDescriptor(
  name='LaneletType',
  full_name='commonroad.LaneletTypeEnum.LaneletType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='URBAN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COUNTRY', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HIGHWAY', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DRIVE_WAY', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAIN_CARRIAGE_WAY', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACCESS_RAMP', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXIT_RAMP', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHOULDER', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS_LANE', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS_STOP', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BICYCLE_LANE', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SIDEWALK', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CROSSWALK', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERSTATE', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERSECTION', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BORDER', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARKING', index=16, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESTRICTED', index=17, number=17,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=18, number=18,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=239,
  serialized_end=536,
)
_sym_db.RegisterEnumDescriptor(_LANELETTYPEENUM_LANELETTYPE)

_ROADUSERENUM_ROADUSER = _descriptor.EnumDescriptor(
  name='RoadUser',
  full_name='commonroad.RoadUserEnum.RoadUser',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='VEHICLE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CAR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRUCK', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRIORITY_VEHICLE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MOTORCYCLE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BICYCLE', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PEDESTRIAN', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRAIN', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TAXI', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=556,
  serialized_end=696,
)
_sym_db.RegisterEnumDescriptor(_ROADUSERENUM_ROADUSER)


_LINEMARKINGENUM = _descriptor.Descriptor(
  name='LineMarkingEnum',
  full_name='commonroad.LineMarkingEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LINEMARKINGENUM_LINEMARKING,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=160,
)


_DRIVINGDIRENUM = _descriptor.Descriptor(
  name='DrivingDirEnum',
  full_name='commonroad.DrivingDirEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DRIVINGDIRENUM_DRIVINGDIR,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=162,
  serialized_end=216,
)


_LANELETTYPEENUM = _descriptor.Descriptor(
  name='LaneletTypeEnum',
  full_name='commonroad.LaneletTypeEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LANELETTYPEENUM_LANELETTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=219,
  serialized_end=536,
)


_ROADUSERENUM = _descriptor.Descriptor(
  name='RoadUserEnum',
  full_name='commonroad.RoadUserEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ROADUSERENUM_ROADUSER,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=539,
  serialized_end=696,
)


_BOUND = _descriptor.Descriptor(
  name='Bound',
  full_name='commonroad.Bound',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='commonroad.Bound.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='line_marking', full_name='commonroad.Bound.line_marking', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=698,
  serialized_end=803,
)


_STOPLINE = _descriptor.Descriptor(
  name='StopLine',
  full_name='commonroad.StopLine',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='commonroad.StopLine.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='line_marking', full_name='commonroad.StopLine.line_marking', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_sign_refs', full_name='commonroad.StopLine.traffic_sign_refs', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_light_refs', full_name='commonroad.StopLine.traffic_light_refs', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=806,
  serialized_end=969,
)


_LANELET = _descriptor.Descriptor(
  name='Lanelet',
  full_name='commonroad.Lanelet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lanelet_id', full_name='commonroad.Lanelet.lanelet_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left_bound', full_name='commonroad.Lanelet.left_bound', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right_bound', full_name='commonroad.Lanelet.right_bound', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='predecessors', full_name='commonroad.Lanelet.predecessors', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors', full_name='commonroad.Lanelet.successors', index=4,
      number=5, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_left', full_name='commonroad.Lanelet.adjacent_left', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_right', full_name='commonroad.Lanelet.adjacent_right', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_left_dir', full_name='commonroad.Lanelet.adjacent_left_dir', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_right_dir', full_name='commonroad.Lanelet.adjacent_right_dir', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stop_line', full_name='commonroad.Lanelet.stop_line', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lanelet_types', full_name='commonroad.Lanelet.lanelet_types', index=10,
      number=11, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_one_way', full_name='commonroad.Lanelet.user_one_way', index=11,
      number=12, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_bidirectional', full_name='commonroad.Lanelet.user_bidirectional', index=12,
      number=13, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_sign_refs', full_name='commonroad.Lanelet.traffic_sign_refs', index=13,
      number=14, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_light_refs', full_name='commonroad.Lanelet.traffic_light_refs', index=14,
      number=15, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=972,
  serialized_end=1582,
)

_LINEMARKINGENUM_LINEMARKING.containing_type = _LINEMARKINGENUM
_DRIVINGDIRENUM_DRIVINGDIR.containing_type = _DRIVINGDIRENUM
_LANELETTYPEENUM_LANELETTYPE.containing_type = _LANELETTYPEENUM
_ROADUSERENUM_ROADUSER.containing_type = _ROADUSERENUM
_BOUND.fields_by_name['points'].message_type = util__pb2._POINT
_BOUND.fields_by_name['line_marking'].enum_type = _LINEMARKINGENUM_LINEMARKING
_STOPLINE.fields_by_name['points'].message_type = util__pb2._POINT
_STOPLINE.fields_by_name['line_marking'].enum_type = _LINEMARKINGENUM_LINEMARKING
_LANELET.fields_by_name['left_bound'].message_type = _BOUND
_LANELET.fields_by_name['right_bound'].message_type = _BOUND
_LANELET.fields_by_name['adjacent_left_dir'].enum_type = _DRIVINGDIRENUM_DRIVINGDIR
_LANELET.fields_by_name['adjacent_right_dir'].enum_type = _DRIVINGDIRENUM_DRIVINGDIR
_LANELET.fields_by_name['stop_line'].message_type = _STOPLINE
_LANELET.fields_by_name['lanelet_types'].enum_type = _LANELETTYPEENUM_LANELETTYPE
_LANELET.fields_by_name['user_one_way'].enum_type = _ROADUSERENUM_ROADUSER
_LANELET.fields_by_name['user_bidirectional'].enum_type = _ROADUSERENUM_ROADUSER
DESCRIPTOR.message_types_by_name['LineMarkingEnum'] = _LINEMARKINGENUM
DESCRIPTOR.message_types_by_name['DrivingDirEnum'] = _DRIVINGDIRENUM
DESCRIPTOR.message_types_by_name['LaneletTypeEnum'] = _LANELETTYPEENUM
DESCRIPTOR.message_types_by_name['RoadUserEnum'] = _ROADUSERENUM
DESCRIPTOR.message_types_by_name['Bound'] = _BOUND
DESCRIPTOR.message_types_by_name['StopLine'] = _STOPLINE
DESCRIPTOR.message_types_by_name['Lanelet'] = _LANELET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LineMarkingEnum = _reflection.GeneratedProtocolMessageType('LineMarkingEnum', (_message.Message,), {
  'DESCRIPTOR' : _LINEMARKINGENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.LineMarkingEnum)
  })
_sym_db.RegisterMessage(LineMarkingEnum)

DrivingDirEnum = _reflection.GeneratedProtocolMessageType('DrivingDirEnum', (_message.Message,), {
  'DESCRIPTOR' : _DRIVINGDIRENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.DrivingDirEnum)
  })
_sym_db.RegisterMessage(DrivingDirEnum)

LaneletTypeEnum = _reflection.GeneratedProtocolMessageType('LaneletTypeEnum', (_message.Message,), {
  'DESCRIPTOR' : _LANELETTYPEENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.LaneletTypeEnum)
  })
_sym_db.RegisterMessage(LaneletTypeEnum)

RoadUserEnum = _reflection.GeneratedProtocolMessageType('RoadUserEnum', (_message.Message,), {
  'DESCRIPTOR' : _ROADUSERENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.RoadUserEnum)
  })
_sym_db.RegisterMessage(RoadUserEnum)

Bound = _reflection.GeneratedProtocolMessageType('Bound', (_message.Message,), {
  'DESCRIPTOR' : _BOUND,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Bound)
  })
_sym_db.RegisterMessage(Bound)

StopLine = _reflection.GeneratedProtocolMessageType('StopLine', (_message.Message,), {
  'DESCRIPTOR' : _STOPLINE,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.StopLine)
  })
_sym_db.RegisterMessage(StopLine)

Lanelet = _reflection.GeneratedProtocolMessageType('Lanelet', (_message.Message,), {
  'DESCRIPTOR' : _LANELET,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Lanelet)
  })
_sym_db.RegisterMessage(Lanelet)


# @@protoc_insertion_point(module_scope)



import enum
from typing import List
import numpy as np

import commonroad.geometry.transform
from commonroad.visualization.draw_params import OptionalSpecificOrAllDrawParams, TrafficLightParams
from commonroad.visualization.drawable import IDrawable
from commonroad.visualization.renderer import IRenderer
from commonroad.common.validity import *


@enum.unique
class TrafficLightState(enum.Enum):
    """
    Enum for the possible types of traffic light in signals
    """
    RED = "red"
    YELLOW = "yellow"
    RED_YELLOW = "redYellow"
    GREEN = "green"
    INACTIVE = "inactive"


@enum.unique
class TrafficLightDirection(enum.Enum):
    """
    Enum for all the possible directions for a traffic signal
    """
    RIGHT = "right"
    STRAIGHT = "straight"
    LEFT = "left"
    LEFT_STRAIGHT = "leftStraight"
    STRAIGHT_RIGHT = "straightRight"
    LEFT_RIGHT = "leftRight"
    ALL = "all"


class TrafficLightCycleElement:
    """Class to represent a traffic light cycle"""

    def __init__(self, state: TrafficLightState, duration: int):
        """
        :param state: state of a traffic light cycle element
        :param duration: duration of traffic light cycle element
        """
        self._state = state
        self._duration = duration

    def __eq__(self, other):
        if not isinstance(other, TrafficLightCycleElement):
            warnings.warn(f"Inequality between TrafficLightCycleElement {repr(self)} and different type {type(other)}")
            return False

        if self._state == other.state and self._duration == other.duration:
            return True

        warnings.warn(f"Inequality of TrafficLightCycleElement {repr(self)} and the other one {repr(other)}")
        return False

    def __hash__(self):
        return hash((self._state, self._duration))

    def __str__(self):
        return f"TrafficLightCycleElement with state {self._state} and duration {self._duration}"

    def __repr__(self):
        return f"TrafficLightCycleElement(state={self._state}, duration={self._duration})"

    @property
    def state(self) -> TrafficLightState:
        return self._state

    @state.setter
    def state(self, state: TrafficLightState):
        self._state = state

    @property
    def duration(self) -> int:
        return self._duration

    @duration.setter
    def duration(self, duration: int):
        self._duration = duration


class TrafficLightCycle:
    """ Class to represent a traffic light cycle"""
    def __init__(self, cycle_elements: List[TrafficLightCycleElement] = None,
                 time_offset: int = 0, active: bool = True):
        """
        :param cycle_elements: list of traffic light cycle elements
        :param time_offset: offset of the traffic light cycle
        :param active: boolean indicating if the traffic light is currently active
        """
        if cycle_elements is None:
            self._cycle_elements = []
        else:
            self._cycle_elements = cycle_elements
        self._time_offset = time_offset
        self._active = active

    def __hash__(self):
        return hash((frozenset(self._cycle_elements), self._time_offset, self._active))

    def __eq__(self, other):
        if not isinstance(other, TrafficLightCycle):
            warnings.warn(f"Inequality between TrafficLightCycle {repr(self)} and different type {type(other)}")
            return False

        if self._cycle_elements == other.cycle_elements and \
           self._time_offset == other.time_offset and self._active == other.active:
            return True

        warnings.warn(f"Inequality of TrafficLightCycle {repr(self)} and the other one {repr(other)}")
        return False

    def __str__(self):
        return f"TrafficLightCycle element with " \
               f"cycle_elements={self._cycle_elements}, time_offset={self._time_offset} and active={self._active}"

    def __repr__(self):
        return f"TrafficLightCycle(cycle_elements={self._cycle_elements}," \
               f" time_offset={self._time_offset}, active={self._active}"

    @property
    def cycle_elements(self) -> Union[None, List[TrafficLightCycleElement]]:
        """ List of traffic light cycle elements."""
        return self._cycle_elements

    @cycle_elements.setter
    def cycle_elements(self, cycle_elements: Union[None, List[TrafficLightCycleElement]]):
        self._cycle_elements = cycle_elements

    @property
    def time_offset(self) -> int:
        """ Offset of the traffic light cycle."""
        return self._time_offset

    @time_offset.setter
    def time_offset(self, time_offset: int):
        self._time_offset = time_offset

    @property
    def active(self) -> bool:
        """ Boolean indicating if the traffic light is currently active."""
        return self._active

    @active.setter
    def active(self, active: bool):
        self._active = active

    @property
    def cycle_init_timesteps(self):
        if not hasattr(self, '_cycle_init_timesteps'):
            durations = [cycle_el.duration for cycle_el in self._cycle_elements]
            self._cycle_init_timesteps = np.cumsum(durations) + self.time_offset
            self._cycle_init_timesteps = np.insert(self._cycle_init_timesteps, 0, self.time_offset)
        return self._cycle_init_timesteps

    def get_state_at_time_step(self, time_step: int) -> TrafficLightState:
        time_step_mod = ((time_step - self.time_offset) % (
                    self.cycle_init_timesteps[-1] - self.time_offset)) + self.time_offset
        i_cycle = np.argmax(time_step_mod < self.cycle_init_timesteps) - 1
        return self.cycle_elements[i_cycle].state


class TrafficLight(IDrawable):
    """ Class to represent a traffic light"""

    def __init__(self, traffic_light_id: int, position: np.ndarray, traffic_light_cycle: TrafficLightCycle = None,
                 color: List[TrafficLightState] = None, active: bool = True,
                 direction: TrafficLightDirection = TrafficLightDirection.ALL):
        """
        :param traffic_light_id: ID of the traffic light
        :param position: position of the traffic light
        :param color: list of traffic light color light states
        :param traffic_light_cycle: traffic light cycle of the traffic light
        :param active: boolean indicating if the traffic light is currently active
        :param direction: driving directions for which the traffic light is valid
        """
        self._traffic_light_id = traffic_light_id
        self._position = position
        if color is None:
            self._color = []
        else:
            self._color = color
        self._traffic_light_cycle = traffic_light_cycle
        self._active = active
        self._direction = direction

    def __eq__(self, other):
        if not isinstance(other, TrafficLight):
            warnings.warn(f"Inequality between TrafficLight {repr(self)} and different type {type(other)}")
            return False

        position_string = np.array2string(np.around(self._position.astype(float), 10), precision=10)
        position_other_string = np.array2string(np.around(other.position.astype(float), 10), precision=10)

        if self._traffic_light_id == other.traffic_light_id and position_string == position_other_string \
                and self._color == other.color and self._direction == other.direction and \
                self._traffic_light_cycle == other.traffic_light_cycle and self._active == other.active:
            return True

        warnings.warn(f"Inequality of TrafficLight {repr(self)} and the other one {repr(other)}")
        return False

    def __hash__(self):
        position_string = np.array2string(np.around(self._position.astype(float), 10), precision=10)
        return hash((self._traffic_light_id, position_string, self._traffic_light_cycle, frozenset(self._color),
                     self._active, self._direction))

    def __str__(self):
        return f"TrafficLight with id {self._traffic_light_id} placed at {self._position}"

    def __repr__(self):
        return f"TrafficLight(traffic_light_id={self._traffic_light_id}, " \
               f"position={self._position.tolist()}, " \
               f"traffic light cycle={self._traffic_light_cycle}, " \
               f"color={self._color}, " \
               f"active={self._active}, "\
               f"direction={self._direction}"

    @property
    def traffic_light_id(self) -> int:
        """ ID of the traffic light."""
        return self._traffic_light_id

    @traffic_light_id.setter
    def traffic_light_id(self, traffic_light_id: int):
        assert isinstance(traffic_light_id, int), '<TrafficLight/traffic_light_id>: ' \
                                                  'Provided traffic_light_id is not valid! ' \
                                                  'id={}'.format(traffic_light_id)
        self._traffic_light_id = traffic_light_id

    @property
    def position(self) -> np.ndarray:
        """ Position of the traffic light."""
        return self._position

    @position.setter
    def position(self, position: np.ndarray):
        self._position = position

    @property
    def traffic_light_cycle(self) -> Union[None, TrafficLightCycle]:
        """ Traffic light cycle of the traffic light."""
        return self._traffic_light_cycle

    @traffic_light_cycle.setter
    def traffic_light_cycle(self, traffic_light_cycle: Union[None, TrafficLightCycle]):
        self._traffic_light_cycle = traffic_light_cycle

    @property
    def color(self) -> List[TrafficLightState]:
        """ List of traffic light color light states."""
        return self._color

    @color.setter
    def color(self, color: List[TrafficLightState]):
        self._color = color

    @property
    def active(self) -> Union[None, bool]:
        """ Boolean indicating if the traffic light is currently active."""
        return self._active

    @active.setter
    def active(self, active: Union[None, bool]):
        self._active = active

    @property
    def direction(self) -> TrafficLightDirection:
        """ Driving directions for which the traffic light is valid."""
        return self._direction

    @direction.setter
    def direction(self, direction: TrafficLightDirection):
        self._direction = direction

    def translate_rotate(self, translation: np.ndarray, angle: float):
        """
        This method translates and rotates a traffic light

        :param translation: The translation given as [x_off,y_off] for the x and y translation
        :param angle: The rotation angle in radian (counter-clockwise defined)
        """

        assert is_real_number_vector(translation, 2), '<TrafficLight/translate_rotate>: ' \
                                                      'argument translation is ' \
                                                      'not a vector of real numbers of ' \
                                                      'length 2.'
        assert is_real_number(angle), '<TrafficLight/translate_rotate>: argument angle must ' \
                                      'be ' \
                                      'a scalar. ' \
                                      'angle = %s' % angle
        assert is_valid_orientation(angle), '<TrafficLight/translate_rotate>: argument angle must ' \
                                            'be ' \
                                            'within the ' \
                                            'interval [-2pi, 2pi]. angle = %s' % angle
        self._position = commonroad.geometry.transform.translate_rotate(np.array([self._position]), translation, angle)[
            0]

    def draw(self, renderer: IRenderer, draw_params: OptionalSpecificOrAllDrawParams[TrafficLightParams] = None):
        renderer.draw_traffic_light_sign(self, draw_params)

    def get_state_at_time_step(self, time_step: int) -> TrafficLightState:
        return self.traffic_light_cycle.get_state_at_time_step(time_step)



from abc import ABC, abstractmethod
from typing import Tuple

from commonroad.common.util import Path_T
from commonroad.planning.planning_problem import PlanningProblemSet
from commonroad.scenario.lanelet import LaneletNetwork
from commonroad.scenario.scenario import Scenario


class FileReader(ABC):
    """
    Interface for reading CommonRoad files in a specific format.
    """

    def __init__(self, filename: Path_T):
        self._filename = filename

    @abstractmethod
    def open(self, lanelet_assignment: bool = False) -> Tuple[Scenario, PlanningProblemSet]:
        pass

    @abstractmethod
    def open_lanelet_network(self) -> LaneletNetwork:
        pass



import dataclasses
import inspect
from dataclasses import dataclass, field
import pathlib
from typing import Any, Optional, Dict, TypeVar, Union, List

from omegaconf import OmegaConf

Color = str


def _dict_to_params(dict_params, cls):
    fields = dataclasses.fields(cls)
    cls_map = {f.name: f.type for f in fields}
    kwargs = {}
    for k, v in cls_map.items():
        if k not in dict_params:
            continue
        if inspect.isclass(v) and issubclass(v, BaseParam):
            kwargs[k] = _dict_to_params(dict_params[k], cls_map[k])
        else:
            kwargs[k] = dict_params[k]
    return cls(**kwargs)


@dataclass
class BaseParam:
    #: First time step of the visualized time interval.
    time_begin: int = 0
    #: Last time step of the visualized time interval.
    time_end: int = 200
    #: Use anti-aliasing.
    antialiased: bool = True
    __initialized: bool = field(init=False, default=False, repr=False)

    def __post_init__(self):
        self.__initialized = True
        # Make sure that the base parameters are propagated to all sub-parameters
        # This cannot be done in the init method, because the sub-parameters are not yet initialized.
        # This is not a noop, as it calls the __setattr__ method.
        # Do not remove!
        self.time_begin = self.time_begin
        self.time_end = self.time_end
        self.antialiased = self.antialiased

    def __setattr__(self, name: str, value: Any) -> None:
        if name in {f.name for f in dataclasses.fields(self)}:
            super().__setattr__(name, value)
        if self.__initialized:
            for k, v in self.__dict__.items():
                if isinstance(v, BaseParam):
                    v.__setattr__(name, value)

    def __getitem__(self, item):
        try:
            value = self.__getattribute__(item)
        except AttributeError:
            raise KeyError(f"{item} is not a parameter of {self.__class__.__name__}")
        return value

    def __setitem__(self, key, value):
        try:
            self.__setattr__(key, value)
        except AttributeError:
            raise KeyError(f"{key} is not a parameter of {self.__class__.__name__}")

    @classmethod
    def load(cls, file_path: Union[pathlib.Path, str], validate_types: bool = True):
        file_path = pathlib.Path(file_path)
        assert file_path.suffix == ".yaml", f"File type {file_path.suffix} is unsupported! Please use .yaml!"
        loaded_yaml = OmegaConf.load(file_path)
        if validate_types:
            OmegaConf.merge(OmegaConf.structured(MPDrawParams), loaded_yaml)
        params = _dict_to_params(OmegaConf.to_object(loaded_yaml), cls)
        return params

    def save(self, file_path: Union[pathlib.Path, str]):
        # Avoid saving private attributes
        dict_cfg = dataclasses.asdict(self, dict_factory=lambda items: {key: val for key, val in items if
                                                                        not key.startswith("_")})
        OmegaConf.save(OmegaConf.create(dict_cfg), file_path, resolve=True)


@dataclass
class ShapeParams(BaseParam):
    opacity: float = 1.0
    facecolor: Color = "#1d7eea"
    edgecolor: Color = "#00478f"
    linewidth: float = 0.5
    zorder: float = 20
    # draw mesh of Polygon
    # NOTE: This parameter is currently only valid for Collision Polygons created by the CommonRoad-Drivability-Checker
    # and has no effect for the Polygon class defined in commonroad-io.geometry.shape
    draw_mesh: bool = False


@dataclass
class VehicleSignalParams(BaseParam):
    signal_radius: float = 0.5
    indicator: ShapeParams = field(default_factory=lambda: ShapeParams(facecolor="#ebc200", zorder=20.2, linewidth=0.0))
    braking: ShapeParams = field(default_factory=lambda: ShapeParams(facecolor="red", zorder=20.2, linewidth=0.0))
    horn: ShapeParams = field(default_factory=lambda: ShapeParams(facecolor="red", zorder=20.1, linewidth=0.0))
    bluelight: ShapeParams = field(default_factory=lambda: ShapeParams(facecolor="blue", zorder=20.1, linewidth=0.0))


@dataclass
class ArrowParams(BaseParam):
    linewidth: float = 1.5
    edgecolor: Color = "black"
    facecolor: Color = "black"
    width: float = 0.8


@dataclass
class StateParams(BaseParam):
    draw_arrow: bool = False
    radius: float = 0.5
    scale_factor: float = 0.3
    linewidth: Optional[float] = None
    edgecolor: Optional[Color] = "black"
    facecolor: Optional[Color] = "black"
    zorder: float = 25
    arrow: ArrowParams = field(default_factory=ArrowParams)


@dataclass
class OccupancyParams(BaseParam):
    draw_occupancies: bool = True
    shape: ShapeParams = field(default_factory=lambda: ShapeParams(zorder=18, opacity=0.2))
    uncertain_position: ShapeParams = field(
            default_factory=lambda: ShapeParams(opacity=0.6, facecolor="k", edgecolor="r"))


@dataclass
class HistoryParams(BaseParam):
    """Draw the history of an object with fading colors."""
    draw_history: bool = False
    steps: int = 5
    step_size: int = 1
    fade_color: float = 0.1
    basecolor: Color = "#ffe119"
    occupancy: ShapeParams = field(
            default_factory=lambda: ShapeParams(opacity=0.2, edgecolor="k", linewidth=0.0, zorder=17))


@dataclass
class StaticObstacleParams(BaseParam):
    occupancy: OccupancyParams = field(
            default_factory=lambda: OccupancyParams(shape=ShapeParams(facecolor="#d95558", edgecolor="#831d20")))


@dataclass
class EnvironmentObstacleParams(BaseParam):
    occupancy: OccupancyParams = field(
            default_factory=lambda: OccupancyParams(shape=ShapeParams(facecolor="#0e6000", edgecolor="#831d20")))


@dataclass
class TrafficLightParams(BaseParam):
    draw_traffic_lights: bool = True
    #: Color of the red phase
    red_color: Color = "red"
    #: Color of the yellow phase
    yellow_color: Color = "#feb609"
    #: Color of the green phase
    green_color: Color = "#00aa16"
    #: Color of the red-yellow phase
    red_yellow_color: Color = "#fe4009ff"
    #: Show id of the traffic light
    show_label: bool = False
    scale_factor: float = 1.0
    zorder: float = 30


@dataclass
class TrafficSignParams(BaseParam):
    #: Enable drawing of traffic signs
    draw_traffic_signs: bool = False
    #: Limit traffic signs to a list of IDs, or None to include all IDs.
    show_traffic_signs: Optional[List[int]] = None
    #: Select mph or kmh as unit of the speed limit. Choose "auto" to determine based on the country.
    speed_limit_unit: str = "auto"
    #: Show id of the traffic sign
    show_label: bool = False
    scale_factor: float = 1.0
    zorder: float = 30


@dataclass
class IntersectionParams(BaseParam):
    draw_intersections: bool = False
    draw_incoming_lanelets: bool = True
    incoming_lanelets_color: Color = "#3ecbcf"
    draw_crossings: bool = True
    crossings_color: Color = "#b62a55"
    draw_successors: bool = True
    successors_left_color: Color = "#ff00ff"
    successors_straight_color: Color = "blue"
    successors_right_color: Color = "#ccff00"
    #: Show the ID of the intersection
    show_label: bool = False


@dataclass
class LaneletParams(BaseParam):
    left_bound_color: Color = "#555555"
    right_bound_color: Color = "#555555"
    center_bound_color: Color = "#dddddd"
    unique_colors: bool = False
    draw_stop_line: bool = True
    stop_line_color: Color = "#ffffff"
    draw_line_markings: bool = True
    draw_left_bound: bool = True
    draw_right_bound: bool = True
    draw_center_bound: bool = True
    draw_border_vertices: bool = False
    #: Show a triangle at the beginning of the lanelet, pointing in the driving direction.
    draw_start_and_direction: bool = True
    #: Apply a colormap to the center line of the lanelet, according the tangent angle.
    colormap_tangent: bool = False
    #: Show the ID of the lanelet.
    show_label: bool = False
    draw_linewidth: float = 0.5
    #: Fill the lanelet with a solid color.
    fill_lanelet: bool = True
    #: Fill color of the lanelet.
    facecolor: Color = "#c7c7c7"


@dataclass
class LaneletNetworkParams(BaseParam):
    kwargs_traffic_light_signs: Dict[str, Any] = field(default_factory=dict)
    #: Limit lanelets to a list of IDs, or None to include all IDs.
    draw_ids: Optional[List[int]] = None
    #: Options for lanelet drawing.
    lanelet: LaneletParams = field(default_factory=LaneletParams)
    #: Options for intersection drawing.
    intersection: IntersectionParams = field(default_factory=IntersectionParams)
    #: Offset angle for tangent colormap. See LaneletParams.colormap_tangent
    relative_angle: float = 0.0
    #: Options for traffic sign drawing.
    traffic_sign: TrafficSignParams = field(default_factory=TrafficSignParams)
    #: Options for traffic light drawing.
    traffic_light: TrafficLightParams = field(default_factory=TrafficLightParams)


@dataclass
class TrajectoryParams(BaseParam):
    draw_trajectory: bool = True
    facecolor: Color = "k"
    #: Draw trajectories as a continuous line instead of dots.
    draw_continuous: bool = False
    line_width: float = 0.17
    #: Use unique colors for each of the trajectories points.
    unique_colors: bool = False
    #: Parameters for shapes of uncertain position.
    shape: ShapeParams = field(default_factory=lambda: ShapeParams(facecolor="k", edgecolor="r", zorder=24))
    zorder: float = 24


@dataclass
class InitialStateParams(BaseParam):
    label_zorder: float = 35
    label: str = ""
    state: StateParams = field(
            default_factory=lambda: StateParams(draw_arrow=True, radius=1.0, scale_factor=0.5, facecolor="g", zorder=11,
                                                arrow=ArrowParams(facecolor="g", edgecolor="g")))


@dataclass
class PlanningProblemParams(BaseParam):
    initial_state: InitialStateParams = field(default_factory=InitialStateParams)
    goal_region: OccupancyParams = field(
        default_factory=lambda: OccupancyParams(shape=ShapeParams(facecolor="#f1b514", edgecolor="#302404", zorder=15)))
    #: Goal lanelet parameters
    lanelet: LaneletParams = field(default_factory=LaneletParams)


@dataclass
class PlanningProblemSetParams(BaseParam):
    #: Limit planning problems to a list of IDs, or None to include all IDs.
    draw_ids: Optional[List[int]] = None
    planning_problem: PlanningProblemParams = field(default_factory=PlanningProblemParams)


@dataclass
class VehicleShapeParams(BaseParam):
    #: Options for visualizing the direction indicator of the vehicle shape.
    direction: ShapeParams = field(default_factory=lambda: ShapeParams(facecolor="#00478f"))
    #: Options for visualizing the vehicle's occupancy in the current time step.
    occupancy: OccupancyParams = field(
            default_factory=lambda: OccupancyParams(shape=ShapeParams(opacity=1.0, zorder=20)))


@dataclass
class DynamicObstacleParams(BaseParam):
    draw_shape: bool = True
    #: Draw a type-dependent icon (if available) instead of the primitive geometric shape.
    draw_icon: bool = False
    #: Draw the direction indicator of the dynamic obstacle.
    draw_direction: bool = False
    draw_bounding_box: bool = True
    #: Show the ID of the dynamic obstacle.
    show_label: bool = False
    #: Visualize the dynamic obstacle signals like indicator or braking lights.
    draw_signals: bool = True
    #: Draw the initial state of the dynamic obstacle.
    draw_initial_state: bool = False
    #: Options for visualizing the vehicle in the current time step.
    vehicle_shape: VehicleShapeParams = field(default_factory=VehicleShapeParams)
    #: Options for visualizing the vehicle signals.
    signals: VehicleSignalParams = field(default_factory=VehicleSignalParams)
    #: Options for visualizing the vehicle states within [time_begin, time_end].
    state: StateParams = field(default_factory=StateParams)
    #: Options for visualizing the vehicle occupancy history with fading colors.
    history: HistoryParams = field(default_factory=HistoryParams)
    #: Options for visualizing the vehicle occupancy in future time steps.
    occupancy: OccupancyParams = field(default_factory=OccupancyParams)
    #: Options for visualizing the vehicle trajectory in future time steps.
    trajectory: TrajectoryParams = field(default_factory=TrajectoryParams)


@dataclass
class PhantomObstacleParams(BaseParam):
    draw_shape: bool = True
    draw_icon: bool = False
    draw_direction: bool = False
    draw_bounding_box: bool = True
    show_label: bool = False
    zorder: float = 20
    draw_signals: bool = True
    vehicle_shape: VehicleShapeParams = field(
        default_factory=lambda: VehicleShapeParams(occupancy=OccupancyParams(shape=ShapeParams(facecolor="#e59dff"))))
    signals: VehicleSignalParams = field(default_factory=VehicleSignalParams)
    draw_initial_state: bool = False
    state: StateParams = field(default_factory=StateParams)
    history: HistoryParams = field(default_factory=HistoryParams)
    occupancy: OccupancyParams = field(default_factory=lambda: OccupancyParams(shape=ShapeParams(facecolor="#c641f6")))
    trajectory: TrajectoryParams = field(default_factory=TrajectoryParams)


@dataclass
class MPDrawParams(BaseParam):
    #: Enable axes for matplotlib.
    axis_visible: bool = True
    shape: ShapeParams = field(default_factory=ShapeParams)
    dynamic_obstacle: DynamicObstacleParams = field(default_factory=DynamicObstacleParams)
    static_obstacle: StaticObstacleParams = field(default_factory=StaticObstacleParams)
    phantom_obstacle: PhantomObstacleParams = field(default_factory=PhantomObstacleParams)
    environment_obstacle: EnvironmentObstacleParams = field(default_factory=EnvironmentObstacleParams)
    trajectory: TrajectoryParams = field(default_factory=TrajectoryParams)
    lanelet_network: LaneletNetworkParams = field(default_factory=LaneletNetworkParams)
    traffic_light: TrafficLightParams = field(default_factory=TrafficLightParams)
    traffic_sign: TrafficSignParams = field(default_factory=TrafficSignParams)
    occupancy: OccupancyParams = field(default_factory=OccupancyParams)
    state: StateParams = field(default_factory=StateParams)
    planning_problem: PlanningProblemParams = field(default_factory=PlanningProblemParams)
    planning_problem_set: PlanningProblemSetParams = field(default_factory=PlanningProblemSetParams)
    initial_state: InitialStateParams = field(default_factory=InitialStateParams)
    goal_region: OccupancyParams = field(default_factory=OccupancyParams)


T = TypeVar("T")
OptionalSpecificOrAllDrawParams = Optional[Union[T, MPDrawParams]]



from typing import Union, Set

from commonroad.common.util import FileFormat
from commonroad.common.writer.file_writer_interface import OverwriteExistingFile
from commonroad.common.writer.file_writer_protobuf import ProtobufFileWriter
from commonroad.common.writer.file_writer_xml import XMLFileWriter
from commonroad.planning.planning_problem import PlanningProblemSet
from commonroad.scenario.scenario import Scenario, Tag, Location


class CommonRoadFileWriter:
    """
    Writes CommonRoad files in XML or protobuf format. The corresponding scenario and planning problem set
    are stored by the writer.
    """

    def __init__(self, scenario: Scenario, planning_problem_set: PlanningProblemSet, author: str = None,
                 affiliation: str = None, source: str = None, tags: Set[Tag] = None, location: Location = None,
                 decimal_precision: int = 4, file_format: FileFormat = FileFormat.XML):
        """
        Initializes the FileWriter for CommonRoad files.

        :param scenario: Scenario
        :param planning_problem_set: Planning problems
        :param author: Name of author
        :param affiliation: Affiliation of author
        :param source: Source of dataset, e.g., database, handcrafted, etc.
        :param tags: Keywords describing the scenario
        :param decimal_precision: Number of decimal places used when writing float values
        :param file_format: Format of file
        :return:
        """
        self._file_format = file_format
        self._file_writer = None
        if file_format == FileFormat.XML:
            self._file_writer = XMLFileWriter(scenario, planning_problem_set, author, affiliation,
                                              source, tags, location, decimal_precision)
        elif file_format == FileFormat.PROTOBUF:
            self._file_writer = ProtobufFileWriter(scenario, planning_problem_set, author, affiliation,
                                                   source, tags, location, decimal_precision)

    def write_to_file(self, filename: Union[str, None] = None,
                      overwrite_existing_file: OverwriteExistingFile = OverwriteExistingFile.ASK_USER_INPUT,
                      check_validity: bool = False):
        """
        Writes CommonRoad scenario and planning problems to file.

        :param filename: Name of file
        :param overwrite_existing_file: Overwriting mode
        :param check_validity: Validity check or not
        :return: Scenario and planning problems
        """
        self._file_writer.write_to_file(filename, overwrite_existing_file, check_validity)

    def write_scenario_to_file(self, filename: Union[str, None] = None,
                               overwrite_existing_file: OverwriteExistingFile = OverwriteExistingFile.ASK_USER_INPUT):
        """
        Writes CommonRoad scenario to file.

        :param filename: Name of file
        :param overwrite_existing_file: Overwriting mode
        :return: Scenario
        """
        self._file_writer.write_scenario_to_file(filename, overwrite_existing_file)

    @staticmethod
    def check_validity_of_commonroad_file(commonroad_str: Union[str, bytes], file_format: FileFormat = FileFormat.XML) \
            -> bool:
        """
        Checks validity of CommonRoad scenario and planning problem stored in XML or protobuf format.

        :param commonroad_str: Commonroad instance stored as string
        :param file_format: Format of file
        :return: Valid or not
        """
        if file_format == FileFormat.XML:
            is_valid = XMLFileWriter.check_validity_of_commonroad_file(commonroad_str)
        else:
            is_valid = ProtobufFileWriter.check_validity_of_commonroad_file(commonroad_str)

        return is_valid



from pathlib import Path
from typing import Tuple, Optional

from commonroad.common.util import FileFormat, Path_T
from commonroad.common.reader.file_reader_protobuf import ProtobufFileReader
from commonroad.common.reader.file_reader_xml import XMLFileReader
from commonroad.planning.planning_problem import PlanningProblemSet
from commonroad.scenario.lanelet import LaneletNetwork
from commonroad.scenario.scenario import Scenario
from commonroad.scenario.traffic_sign import *


class CommonRoadFileReader:
    """
    Reads CommonRoad files in XML or protobuf format. The corresponding stored scenario and planning problem set
    are created by the reader.
    """
    def __init__(self, filename: Path_T, file_format: Optional[FileFormat] = None):
        """
        Initializes the FileReader for CommonRoad files.

        :param filename: Name of file
        :param file_format: Format of file. If None, inferred from file suffix.
        :return:
        """
        self._file_reader = None

        if file_format is None:
            file_format = FileFormat(Path(filename).suffix)

        if file_format == FileFormat.XML:
            self._file_reader = XMLFileReader(filename)
        elif file_format == FileFormat.PROTOBUF:
            self._file_reader = ProtobufFileReader(filename)

    def open(self, lanelet_assignment: bool = False) -> Tuple[Scenario, PlanningProblemSet]:
        """
        Opens and loads CommonRoad scenario and planning problems from file.

        :param lanelet_assignment: Activates calculation of lanelets occupied by obstacles
        :return: Scenario and planning problems
        """
        return self._file_reader.open(lanelet_assignment)

    def open_lanelet_network(self) -> LaneletNetwork:
        """
        Opens and loads CommonRoad lanelet network from file.

        :return:
        """
        return self._file_reader.open_lanelet_network()



import numpy as np
import math
from typing import List, Union


def translate_rotate(vertices: np.ndarray, translation: Union[np.array, List[float]], angle: Union[float, int])\
        -> np.ndarray:
    """
    First translates the list of vertices, then rotates the list of vertices around the origin.

    :param vertices: array of 2D vertices [[x_0, y_0], [x_1, y_1], ...]
    :param translation: translation vector [x_off, y_off] in x- and y-direction
    :param angle: rotation angle in radian (counter-clockwise)
    :return: array of transformed vertices [[x'_0, y'_0], [x'_1, y'_1], ...]
    """

    h_vertices = to_homogeneous_coordinates(vertices)
    return from_homogeneous_coordinates(translation_rotation_matrix(translation, angle).
                                        dot(h_vertices.transpose()).transpose())


def rotate_translate(vertices: np.ndarray, translation: Union[np.array, List[float]], angle: Union[float, int])\
        -> np.ndarray:
    """
    First rotates the list of vertices around the origin and then translates the list of vertices.

    :param vertices: array of 2D vertices [[x_0, y_0], [x_1, y_1], ...]
    :param translation: translation vector [x_off, y_off] in x- and y-direction
    :param angle: rotation angle in radian (counter-clockwise)
    :return: array of transformed vertices [[x'_0, y'_0], [x'_1, y'_1], ...]
    """

    h_vertices = to_homogeneous_coordinates(vertices)
    return from_homogeneous_coordinates(rotation_translation_matrix(translation, angle).
                                        dot(h_vertices.transpose()).transpose())


def rotation_translation_matrix(translation: Union[np.array, List[float]], angle: Union[float, int]) -> np.ndarray:
    """
    Creates a matrix that first rotates a vector around the origin and then translates it.

    :param translation: offset in (x, y) for translating the vector
    :param angle: angle in rad [-2pi, +2pi]
    :return: matrix
    """
    if angle == 0:
        cos_angle = 1.0
        sin_angle = 0.0
    else:
        cos_angle = math.cos(angle)
        sin_angle = math.sin(angle)

    return np.array([[cos_angle, -sin_angle, translation[0]],
                     [sin_angle,  cos_angle, translation[1]],
                     [0, 0, 1]])


def translation_rotation_matrix(translation: Union[np.array, List[float]], angle: Union[float, int]) -> np.ndarray:
    """
    Creates a matrix that first translates a homogeneous point, and then rotates it around the origin.

    :param translation: offset in (x, y) for translating the vector
    :param angle: angle in rad [-2pi, +2pi]
    :return: matrix
    """
    translation_matrix = np.array([[1.0, 0.0, translation[0]],
                                   [0.0, 1.0, translation[1]],
                                   [0.0, 0.0, 1.0]], dtype=np.float64)
    if np.abs(angle) <= 0.05:
        cos_angle = 1.0
        sin_angle = angle
    else:
        cos_angle = math.cos(angle)
        sin_angle = math.sin(angle)

    rotation_matrix = np.array([[cos_angle, -sin_angle, 0.0],
                                [sin_angle, cos_angle, 0.0],
                                [0.0, 0.0, 1.0]], dtype=np.float64)
    return rotation_matrix.dot(translation_matrix)


def to_homogeneous_coordinates(points: np.array) -> np.ndarray:
    """
    Converts an array of vertices to homogeneous coordinates.

    :param points: array of points
    :return: homogeneous points
    """
    assert(len(points.shape) == 2)
    assert(points.shape[1] == 2)
    return np.hstack((points, np.ones((len(points), 1))))


def from_homogeneous_coordinates(points) -> np.ndarray:
    """
    Converts an array of homogeneous vertices back to 2D.

    :param points: array of points
    :return: array of 2D points
    """
    assert(len(points.shape) == 2)
    assert(points.shape[1] == 3)
    return points[:, 0:2]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: intersection.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='intersection.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12intersection.proto\x12\ncommonroad\"\x9e\x01\n\x08Incoming\x12\x13\n\x0bincoming_id\x18\x01 \x02(\r\x12\x19\n\x11incoming_lanelets\x18\x02 \x03(\r\x12\x18\n\x10successors_right\x18\x03 \x03(\r\x12\x1b\n\x13successors_straight\x18\x04 \x03(\r\x12\x17\n\x0fsuccessors_left\x18\x05 \x03(\r\x12\x12\n\nis_left_of\x18\x06 \x01(\r\"k\n\x0cIntersection\x12\x17\n\x0fintersection_id\x18\x01 \x02(\r\x12\'\n\tincomings\x18\x02 \x03(\x0b\x32\x14.commonroad.Incoming\x12\x19\n\x11\x63rossing_lanelets\x18\x03 \x03(\r'
)




_INCOMING = _descriptor.Descriptor(
  name='Incoming',
  full_name='commonroad.Incoming',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='incoming_id', full_name='commonroad.Incoming.incoming_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='incoming_lanelets', full_name='commonroad.Incoming.incoming_lanelets', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors_right', full_name='commonroad.Incoming.successors_right', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors_straight', full_name='commonroad.Incoming.successors_straight', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors_left', full_name='commonroad.Incoming.successors_left', index=4,
      number=5, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_left_of', full_name='commonroad.Incoming.is_left_of', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=35,
  serialized_end=193,
)


_INTERSECTION = _descriptor.Descriptor(
  name='Intersection',
  full_name='commonroad.Intersection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intersection_id', full_name='commonroad.Intersection.intersection_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='incomings', full_name='commonroad.Intersection.incomings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='crossing_lanelets', full_name='commonroad.Intersection.crossing_lanelets', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=195,
  serialized_end=302,
)

_INTERSECTION.fields_by_name['incomings'].message_type = _INCOMING
DESCRIPTOR.message_types_by_name['Incoming'] = _INCOMING
DESCRIPTOR.message_types_by_name['Intersection'] = _INTERSECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Incoming = _reflection.GeneratedProtocolMessageType('Incoming', (_message.Message,), {
  'DESCRIPTOR' : _INCOMING,
  '__module__' : 'intersection_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Incoming)
  })
_sym_db.RegisterMessage(Incoming)

Intersection = _reflection.GeneratedProtocolMessageType('Intersection', (_message.Message,), {
  'DESCRIPTOR' : _INTERSECTION,
  '__module__' : 'intersection_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Intersection)
  })
_sym_db.RegisterMessage(Intersection)


# @@protoc_insertion_point(module_scope)
